#!/usr/bin/env python
#
# Created by: Pearu Peterson, April 2002
#

__usage__ = """
Build linalg:
  python setup_linalg.py build
Run tests if scipy is installed:
  python -c 'import scipy;scipy.linalg.test(<level>)'
Run tests if linalg is not installed:
  python tests/test_blas.py [<level>]
"""


from Numeric import arange, add, array
import math

import sys
from scipy_test.testing import *
set_package_path()
from linalg import fblas
from linalg import cblas
del sys.path[0]

class test_cblas1_simple(ScipyTestCase):

    def check_axpy(self):
        for p in 'sd':
            f = getattr(cblas,p+'axpy',None)
            if f is None: continue
            assert_array_almost_equal(f(5,[1,2,3],[2,-1,3]),[7,9,18])
        for p in 'cz':
            f = getattr(cblas,p+'axpy',None)
            if f is None: continue
            assert_array_almost_equal(f(5,[1,2j,3],[2,-1,3]),[7,10j-1,18])

class test_fblas1_simple(ScipyTestCase):

    def check_axpy(self):
        for p in 'sd':
            f = getattr(fblas,p+'axpy',None)
            if f is None: continue
            assert_array_almost_equal(f([1,2,3],[2,-1,3],a=5),[7,9,18])
        for p in 'cz':
            f = getattr(fblas,p+'axpy',None)
            if f is None: continue
            assert_array_almost_equal(f([1,2j,3],[2,-1,3],a=5),[7,10j-1,18])
    def check_copy(self):
        for p in 'sd':
            f = getattr(fblas,p+'copy',None)
            if f is None: continue
            assert_array_almost_equal(f([3,4,5],[8]*3),[3,4,5])
        for p in 'cz':
            f = getattr(fblas,p+'copy',None)
            if f is None: continue
            assert_array_almost_equal(f([3,4j,5+3j],[8]*3),[3,4j,5+3j])
    def check_asum(self):
        for p in 'sd':
            f = getattr(fblas,p+'asum',None)
            if f is None: continue
            assert_almost_equal(f([3,-4,5]),12)
        for p in ['sc','dz']:
            f = getattr(fblas,p+'asum',None)
            if f is None: continue
            assert_almost_equal(f([3j,-4,3-4j]),14)
    def check_dot(self):
        for p in 'sd':
            f = getattr(fblas,p+'dot',None)
            if f is None: continue
            assert_almost_equal(f([3,-4,5],[2,5,1]),-9)
        for p in 'cz':
            f = getattr(fblas,p+'dotu',None)
            if f is None: continue
            assert_almost_equal(f([3j,-4,3-4j],[2,3,1]),-9+2j)
            f = getattr(fblas,p+'dotc')
            assert_almost_equal(f([3j,-4,3-4j],[2,3j,1]),3-14j)
    def check_nrm2(self):
        for p in 'sd':
            f = getattr(fblas,p+'nrm2',None)
            if f is None: continue
            assert_almost_equal(f([3,-4,5]),math.sqrt(50))
        for p in ['sc','dz']:
            f = getattr(fblas,p+'nrm2',None)
            if f is None: continue
            assert_almost_equal(f([3j,-4,3-4j]),math.sqrt(50))
    def check_scal(self):
        for p in 'sd':
            f = getattr(fblas,p+'scal',None)
            if f is None: continue
            assert_array_almost_equal(f(2,[3,-4,5]),[6,-8,10])
        for p in 'cz':
            f = getattr(fblas,p+'scal',None)
            if f is None: continue
            assert_array_almost_equal(f(3j,[3j,-4,3-4j]),[-9,-12j,12+9j])
        for p in ['cs','zd']:
            f = getattr(fblas,p+'scal',None)
            if f is None: continue
            assert_array_almost_equal(f(3,[3j,-4,3-4j]),[9j,-12,9-12j])
    def check_swap(self):
        for p in 'sd':
            f = getattr(fblas,p+'swap',None)
            if f is None: continue
            x,y = [2,3,1],[-2,3,7]
            x1,y1 = f(x,y)
            assert_array_almost_equal(x1,y)
            assert_array_almost_equal(y1,x)
        for p in 'cz':
            f = getattr(fblas,p+'swap',None)
            if f is None: continue
            x,y = [2,3j,1],[-2,3,7-3j]
            x1,y1 = f(x,y)
            assert_array_almost_equal(x1,y)
            assert_array_almost_equal(y1,x)
    def check_amax(self):
        for p in 'sd':
            f = getattr(fblas,'i'+p+'amax')
            assert_equal(f([-2,4,3]),1)
        for p in 'cz':
            f = getattr(fblas,'i'+p+'amax')
            assert_equal(f([-5,4+3j,6]),1)
    #XXX: need tests for rot,rotm,rotg,rotmg

class test_fblas2_simple(ScipyTestCase):

    def check_gemv(self):
        for p in 'sd':
            f = getattr(fblas,p+'gemv',None)
            if f is None: continue
            assert_array_almost_equal(f(3,[[3]],[-4]),[-36])
            assert_array_almost_equal(f(3,[[3]],[-4],3,[5]),[-21])
        for p in 'cz':
            f = getattr(fblas,p+'gemv',None)
            if f is None: continue
            assert_array_almost_equal(f(3j,[[3-4j]],[-4]),[-48-36j])
            assert_array_almost_equal(f(3j,[[3-4j]],[-4],3,[5j]),[-48-21j])

    def check_ger(self):

        for p in 'sd':
            f = getattr(fblas,p+'ger',None)
            if f is None: continue
            assert_array_almost_equal(f(1,[1,
                                           2],[3,4]),[[3,4],[6,8]])
            assert_array_almost_equal(f(2,[1,
                                           2,
                                           3],[3,4]),[[6,8],[12,16],[18,24]])

            assert_array_almost_equal(f(1,[1,
                                           2],[3,4],
                                        a=[[1,2],[3,4]]
                                        ),[[4,6],[9,12]])

        for p in 'cz':
            f = getattr(fblas,p+'geru',None)
            if f is None: continue
            assert_array_almost_equal(f(1,[1j,
                                           2],[3,4]),[[3j,4j],[6,8]])
            assert_array_almost_equal(f(-2,[1j,
                                           2j,
                                           3j],[3j,4j]),[[6,8],[12,16],[18,24]])

        for p in 'cz':
            f = getattr(fblas,p+'gerc',None)
            if f is None: continue
            assert_array_almost_equal(f(1,[1j,
                                           2],[3,4]),[[3j,4j],[6,8]])
            assert_array_almost_equal(f(2,[1j,
                                           2j,
                                           3j],[3j,4j]),[[6,8],[12,16],[18,24]])

class test_fblas3_simple(ScipyTestCase):

    def check_gemm(self):
        for p in 'sd':
            f = getattr(fblas,p+'gemm',None)
            if f is None: continue
            assert_array_almost_equal(f(3,[3],[-4]),[-36])
            assert_array_almost_equal(f(3,[3],[-4],3,[5]),[-21])
        for p in 'cz':
            f = getattr(fblas,p+'gemm',None)
            if f is None: continue
            assert_array_almost_equal(f(3j,[3-4j],[-4]),[-48-36j])
            assert_array_almost_equal(f(3j,[3-4j],[-4],3,[5j]),[-48-21j])

class test_blas(ScipyTestCase):

    def check_fblas(self):
        if hasattr(fblas,'empty_module'):
            print """
****************************************************************
WARNING: fblas module is empty.
-----------
See scipy/INSTALL.txt for troubleshooting.
****************************************************************
"""
    def check_cblas(self):
        if hasattr(cblas,'empty_module'):
            print """
****************************************************************
WARNING: cblas module is empty
-----------
See scipy/INSTALL.txt for troubleshooting.
Notes:
* If atlas library is not found by scipy/system_info.py,
  then scipy uses fblas instead of cblas.
****************************************************************
"""

if __name__ == "__main__":
    ScipyTest('linalg.blas').run()



"""distutils.extension

Provides the Extension class, used to describe C/C++ extension
modules in setup scripts.

Overridden to support f2py and SourceGenerator.
"""

__revision__ = "$Id: extension.py,v 1.8 2004/03/04 20:14:43 pearu Exp $"

from distutils.extension import Extension as old_Extension
from scipy_distutils.misc_util import SourceGenerator, SourceFilter

import re
cxx_ext_re = re.compile(r'.*[.](cpp|cxx|cc)\Z',re.I).match
fortran_pyf_ext_re = re.compile(r'.*[.](f90|f95|f77|for|ftn|f|pyf)\Z',re.I).match

class Extension(old_Extension):
    def __init__ (self, name, sources,
                  include_dirs=None,
                  define_macros=None,
                  undef_macros=None,
                  library_dirs=None,
                  libraries=None,
                  runtime_library_dirs=None,
                  extra_objects=None,
                  extra_compile_args=None,
                  extra_link_args=None,
                  export_symbols=None,
                  depends=None,
                  language=None,
                  f2py_options=None,
                  module_dirs=None,
                 ):
        old_Extension.__init__(self,name, [],
                               include_dirs,
                               define_macros,
                               undef_macros,
                               library_dirs,
                               libraries,
                               runtime_library_dirs,
                               extra_objects,
                               extra_compile_args,
                               extra_link_args,
                               export_symbols)
        # Avoid assert statements checking that sources contains strings:
        self.sources = sources

        # Python 2.3 distutils new features
        self.depends = depends or []
        self.language = language

        self.f2py_options = f2py_options or []
        self.module_dirs = module_dirs or []

    def has_cxx_sources(self):
        for source in self.sources:
            if isinstance(source,SourceGenerator) \
               or isinstance(source,SourceFilter):
                for s in source.sources:
                    if cxx_ext_re(s):
                        return 1
            if cxx_ext_re(str(source)):
                return 1
        return 0

    def has_f2py_sources(self):
        for source in self.sources:
            if isinstance(source,SourceGenerator) \
               or isinstance(source,SourceFilter):
                for s in source.sources:
                    if fortran_pyf_ext_re(s):
                        return 1
            elif fortran_pyf_ext_re(source):
                return 1
        return 0

    def generate_sources(self):
        new_sources = []
        for source in self.sources:
            if isinstance(source, SourceGenerator):
                new_sources.append(source.generate())
            elif isinstance(source, SourceFilter):
                new_sources.extend(source.filter())
            else:
                new_sources.append(source)
        self.sources = new_sources
                
    def get_sources(self):
        sources = []
        for source in self.sources:
            if isinstance(source,SourceGenerator):
                sources.extend(source.sources)
            elif isinstance(source,SourceFilter):
                sources.extend(source.sources)
            else:
                sources.append(source)
        return sources

# class Extension



from types import StringType
from distutils.command.install import *
from distutils.command.install import install as old_install
from distutils.util import convert_path
from distutils.file_util import write_file
from distutils.errors import DistutilsOptionError
from scipy_distutils import log

#install support for Numeric.pth setup

def _quote_name_when_has_spaces(name):
    if ' ' in name and name[0] not in ['"',"'"]:
        name = '"%s"' % (name)
    return name

class install(old_install):

    def run (self):

        # Obviously have to build before we can install
        if not self.skip_build:
            self.run_command('build')

        # Run all sub-commands (at least those that need to be run)
        for cmd_name in self.get_sub_commands():
            self.run_command(cmd_name)

        if self.path_file:
            self.create_path_file()

        # write list of installed files, if requested.
        if self.record:
            outputs = self.get_outputs()
            if self.root:               # strip any package prefix
                root_len = len(self.root)
                for counter in xrange(len(outputs)):
                    outputs[counter] = outputs[counter][root_len:]
            outputs = map(_quote_name_when_has_spaces,outputs)
            self.execute(write_file,
                         (self.record, outputs),
                         "writing list of installed files to '%s'" %
                         self.record)

        sys_path = map(os.path.normpath, sys.path)
        sys_path = map(os.path.normcase, sys_path)
        install_lib = os.path.normcase(os.path.normpath(self.install_lib))
        if (self.warn_dir and
            not (self.path_file and self.install_path_file) and
            install_lib not in sys_path):
            log.debug(("modules installed to '%s', which is not in "
                       "Python's module search path (sys.path) -- " 
                       "you'll have to change the search path yourself"),
                       self.install_lib)

    # run ()


    def finalize_options (self):
        old_install.finalize_options(self)
        self.install_lib = self.install_libbase
        
    def handle_extra_path (self):
        if self.extra_path is None:
            self.extra_path = self.distribution.extra_path

        if self.extra_path is not None:
            if type(self.extra_path) is StringType:
                self.extra_path = string.split(self.extra_path, ',')
            if len(self.extra_path) == 1:
                path_file = extra_dirs = self.extra_path[0]
            elif len(self.extra_path) == 2:
                (path_file, extra_dirs) = self.extra_path
            else:
                raise DistutilsOptionError, \
                      "'extra_path' option must be a list, tuple, or " + \
                      "comma-separated string with 1 or 2 elements"

            # convert to local form in case Unix notation used (as it
            # should be in setup scripts)
            extra_dirs = convert_path(extra_dirs)

        else:
            path_file = None
            extra_dirs = ''

        # XXX should we warn if path_file and not extra_dirs? (in which
        # case the path file would be harmless but pointless)
        self.path_file = path_file
        self.extra_dirs = ''
        self.pth_file = extra_dirs

    # handle_extra_path ()

    def create_path_file (self):
        filename = os.path.join(self.install_libbase,
                                self.path_file + ".pth")
        if self.install_path_file:
            self.execute(write_file,
                         (filename, [self.pth_file]),
                         "creating %s" % filename)
        else:
            log.warn("path file '%s' not created" % filename)



from scipy_base.fastumath import *
from scipy import *
from scipy_base import limits, display_test
import sys

_have_wx = 0
if not display_test.have_x11() or display_test.try_XOpenDisplay():
    try:
        import wxPython
        _have_wx = 1
    except ImportError,msg:
        print __file__,msg

if _have_wx:
    import wxplt
    import plot_objects
    import gui_thread
    plot_module = wxplt
    plot_class = gui_thread.register(plot_module.plot_frame)

_figure = []
_active = None


def figure(which_one = None):
    global _figure; global _active
    if which_one is None:
        title ='Figure %d' % len(_figure)
        _figure.append(plot_class(title=title))
        _active = _figure[-1]
    elif (type(which_one) == type(1)) or (type(which_one) == type(1.)):
        try:    
            _active = _figure[int(which_one)]
            _active.Raise()
        except IndexError:
            msg = "There are currently only %d active figures" % len(_figure)
            raise IndexError, msg
    elif which_one in _figure:
        _active = which_one
        _active.Raise()
    else:
        try:
            if which_one.__type_hack__ == "plot_canvas":
                _active = which_one
                _figure.append(_active)
                _active.Raise()
            else:
                raise ValueError, "The specified figure or index is not not known"
        except (AtrributeError):
            pass
    fig = current()
    return fig
    
    
def validate_active():
    global _active
    if _active is None: figure()
    try:
        if not _active.proxy_object_alive:
            _active = None
            figure()
    except:
        pass        
    
def current():
    return _active

def redraw():
    validate_active()
    _active.redraw()

def close(which_one = None):
    global _figure; global _active
    if which_one is None:
        try:
            _active.Close()
            _figure.remove(_active)
        except ValueError:
            pass
            
        try: 
            # should make sure the new plot window really exist
            set_new_active()
        except IndexError: _active = None
    elif which_one == 'all':
        for fig in _figure: fig.Close()
        _active = None
    else:
        raise NotImplementedError, "currently close only works with"\
                                   " _active window or 'all'"
        #try: 
        #   _figure.remove(which_one)
        #   which_one.close()
        #except ValueError:
        #   which_one.close() 

def set_new_active():
    # should validate new active here
    try:
        _active = _figure[-1]
    except IndexError:
        _active = None    
        

def _auto_all():
    validate_active()
    _active.x_axis.bounds = ['auto','auto']
    _active.y_axis.bounds = ['auto','auto']
    _active.x_axis.tick_interval = 'auto'
    _active.y_axis.tick_interval = 'auto'    
def autoscale():
    validate_active()
    _auto_all()
    _active.update()

def _an_axis(ax,setting):
    ticks = ax.ticks
    interval = ax.ticks[1]- ax.ticks[0]
    if setting in ['normal','auto']:
        ax.bounds = ['auto','auto']
    elif setting == 'freeze':
        ax.bounds = [axes[0],axes[1]]
        ax.tick_interval = interval
    elif setting in ['tight','fit']:
        ax.bounds = ['fit','fit']
        ax.tick_interval = 'auto'
    else:
        ax.bounds = [setting[0],setting[1]]
        if len(setting) > 2:
            ax.tick_interval = setting[2]
            
def xaxis(rng):
    validate_active()
    _an_axis(_active.x_axis,rng)
    _active.update()
    
def yaxis(rng):
    validate_active()
    _an_axis(_active.y_axis,rng)
    _active.update()

def title(name):
    validate_active()
    _active.title.text = name
    _active.update()

def xtitle(name):
    validate_active()
    _active.x_title.text = name
    _active.update()

def ytitle(name):
    validate_active()
    _active.y_title.text = name
    _active.update()

on = 'on'
off = 'off'
def grid(state=None):
    validate_active()
    if state is None:
        if _active.x_axis.grid_visible in ['on','yes']:
            _active.x_axis.grid_visible = 'off'
            _active.y_axis.grid_visible = 'off'
        else:
            _active.x_axis.grid_visible = 'on'
            _active.y_axis.grid_visible = 'on'
    elif state in ['on','off','yes','no']:
        _active.x_axis.grid_visible = state
        _active.y_axis.grid_visible = state
    else:
        raise ValueError, 'grid argument can be "on","off",'\
                          '"yes","no". Not ' + state        
    _active.update()

def hold(state):
    validate_active()
    if state in ['on','off','yes','no']:
        _active.hold = state
    else:
        raise ValueError, 'holds argument can be "on","off",'\
                          '"yes","no". Not ' + state        
    
def axis(setting):
    validate_active()
    x_ticks = _active.x_axis.ticks
    print type(x_ticks), dir(x_ticks)    
    x_interval = x_ticks[1]- x_ticks[0]
    y_ticks = _active.y_axis.ticks
    x_interval = x_ticks[1]- y_ticks[0]
    axes = array((x_ticks[0],x_ticks[-1],y_ticks[0],y_ticks[-1]),Float)
    # had to use client below cause of __setattr__ troubles in plot_frame
    if setting == 'normal':
        _active.client.aspect_ratio = setting
        _auto_all()
    elif setting == 'equal':
        _active.client.aspect_ratio = setting    
    elif setting == 'freeze':
        _active.x_axis.bounds = [axes[0],axes[1]]
        _active.y_axis.bounds = [axes[2],axes[3]]
        _active.x_axis.tick_interval = x_interval
        _active.x_axis.tick_interval = y_interval        
    elif setting in ['tight','fit']:
        _active.x_axis.bounds = ['fit','fit']
        _active.y_axis.bounds = ['fit','fit']
        _active.x_axis.tick_interval = 'auto'
        _active.x_axis.tick_interval = 'auto'
    else:
        _active.x_axis.bounds = [setting[0],setting[1]]
        _active.y_axis.bounds = [setting[2],setting[3]]
    _active.update()    

def save(file_name,format='png'):
    _active.save(file_name,format)


##########################################################
#----------------- plotting machinery -------------------#
##########################################################

#---- array utilities ------------

def is1D(a):
	as = shape(a)
	if(len(as) == 1):
		return 1
	if(as[0] == 1 or as[1]==1):
		return 1
	return 0
	
def row(a):
	return reshape(asarray(a),[1,-1])
def col(a):
	return reshape(asarray(a),[-1,1])

SizeMismatch = 'SizeMismatch'
SizeError = 'SizeError'
NotImplemented = 'NotImplemented'

#------------ Numerical constants ----------------

# really should do better than this...
BIG = 1e20
SMALL = 1e-20

#------------ plot group parsing -----------------
from types import *

def plot_groups(data):
    remains = data; groups = []
    while len(remains):
        group,remains = get_plot_group(remains)
        groups.append(group)
    return groups        
    
def get_plot_group(data):
    group = ()
    remains = data
    state = 0
    finished = 0
    while(len(remains) > 0 and not finished):
        el = remains[0]
        if(state == 0):
            el = asarray(el)
            state = 1 
        elif(state == 1):
            if(type(el) == StringType):
                finished = 1
            else:
                el = asarray(el)
            state = 2
        elif(state == 2):
            finished = 1
            if(type(el) != StringType):
                break
        try:
            if el.typecode() == 'D':
                print 'warning plotting magnitude of complex values'
                el = abs(el)
        except:
            pass
        group = group + (el,)
        remains = remains[1:]       
    return group, remains           

def hstack(tup):		
	#horizontal stack (column wise)
	return concatenate(tup,1)

def lines_from_group(group):
    lines = []
    plotinfo = ''
    x = group[0]
    ar_num = 1
    if len(group) > 1:
        if type(group[1]) == StringType:
            plotinfo = group[1]
        else:
            ar_num = 2
            y = group[1]
    if len(group) == 3:
        plotinfo = group[2]
    #force 1D arrays to 2D columns
    if is1D(x): 
        x = col(x)
    if ar_num == 2 and is1D(y):  
        y = col(y)
    
    xs = shape(x)            
    if ar_num == 2:  ys = shape(y)
    #test that x and y have compatible shapes
    if ar_num == 2:
        #check that each array has the same number of rows
        if(xs[0] != ys[0] ):
            raise SizeMismatch, ('rows', xs, ys)
        #check that x.cols = y.cols
        #no error x has 1 column
        if(xs[1] > 1 and xs[1] != ys[1]):
            raise SizeMismatch, ('cols', xs, ys)
    
    #plot x against index
    if(ar_num == 1):
        for y_data in transpose(x):
            index = arange(len(y_data))
            pts = hstack(( col(index), col(y_data) ))
            pts = remove_bad_vals(pts)
            line = plot_module.line_object(pts)            
            lines.append(line)
    #plot x vs y                    
    elif(ar_num ==2):
        #x is effectively 1D
        if(xs[1] == 1):
            for y_data in transpose(y):
                pts = hstack(( col(x), col(y_data) ))
                pts = remove_bad_vals(pts)
                line = plot_module.line_object(pts)
                lines.append(line)
        #x is 2D                    
        else:
            x = transpose(x); y = transpose(y)
            for i in range(len(x)):
                pts = hstack(( col(x[i]), col(y[i]) ))
                pts = remove_bad_vals(pts)
                line = plot_module.line_object(pts)
                lines.append(line)
    color,marker,line_type = process_format(plotinfo)
    #print color,marker,line_type
    for line in lines:
        if color != 'auto':
            line.color = 'custom'
            line.set_color(color)
            #print color
        if not marker:
            line.marker_type = 'custom'
            line.markers.visible = 'no'
            #print marker
        elif marker != 'auto':
            line.marker_type = 'custom'
            line.markers.symbol = marker
            line.markers.visible = 'yes'
            #print marker
        if not line_type:
            line.line_type = 'custom'
            line.line.visible = 'no'
        elif line_type != 'auto':
            line.line_type = 'custom'
            line.line.visible = 'yes'
            line.line.style = line_type
            #print line_type
        #print line.markers.visible,    line.line.visible,
    return lines

import re
color_re = re.compile('[ymcrgbwk]')
color_trans = {'y':'yellow','m':'magenta','c':'cyan','r':'red','g':'green',
               'b':'blue', 'w':'white','k':'black'}
# this one isn't quite right               
marker_re = re.compile('[ox+s^v]|(?:[^-])[.]')
marker_trans = {'.':'dot','o':'circle','x':'cross','+':'plus','s':'square',
                '^':'triangle','v':'down_triangle'}

line_re = re.compile('--|-\.|[-:]')
line_trans = {'-':'solid',':':'dot','-.':'dot dash','--':'dash'}
def process_format(format):
    if format == '':
        return 'auto','auto','auto'
    color,marker,line = 'auto',None,None
    m = color_re.findall(format)
    if len(m): color = color_trans[m[0]]
    m = marker_re.findall(format)
    # the -1 takes care of 'r.', etc
    if len(m): marker = marker_trans[m[0][-1]]    
    m = line_re.findall(format)
    if len(m): line = line_trans[m[0]]
    return color,marker,line
    
def remove_bad_vals(x):
    # !! Fix axis order when interface changed.
    # mapping:
    #    NaN -> 0
    #    Inf -> limits.double_max
    #   -Inf -> limits.double_min
    y = nan_to_num(x)    
    big = limits.double_max / 10
    small = limits.double_min / 10
    y = clip(y,small,big)
    return y

def stem(*data):
    if len(data) == 1:
        n = arange(len(data[0]))
        x = data[0]
        ltype = ['b-','mo']
    if len(data) == 2:
        if type(data[1]) is types.StringType:
            ltype = [data[1],'mo']
            n = arange(len(data[0]))
            x = data[0]
        elif type(data[1]) in [types.ListType, types.TupleType]:
            n = arange(len(data[0]))
            x = data[0]            
            ltype = data[1][:2]
        else:
            n = data[0]
            x = data[1]
            ltype = ['b-','mo']
    elif len(data) > 2:
        n = data[0]
        x = data[1]
        ltype = data[2]
        if type(ltype) is types.StringType:
            ltype = [ltype,'mo']
    else:
        raise ValueError, "Invalid input arguments."

    if len(n) != len(x):
        raise SizeMismatch, ('lengths', len(n), len(x))
    # line at zero:
    newdata = []
    newdata.extend([[n[0],n[-1]],[0,0],ltype[0]])

    # stems
    for k in range(len(x)):
        newdata.extend([[n[k],n[k]],[0,x[k]],ltype[0]])

    # circles
    newdata.extend([n,x,ltype[1]])
    keywds = {'fill_style': 'transparent'}
    return plot(*newdata,**keywds)
    
                           
    
def plot(*data,**keywds):
    groups = plot_groups(data)
    lines = []
    for group in groups:
        lines.extend(lines_from_group(group))
        #default to markers being invisible
        #lines[-1].markers.visible = 'no'
    # check for hold here
    for name in plot_objects.poly_marker._attributes.keys():
        value = keywds.get(name)
        if value is not None:
            for k in range(len(lines)):
                exec('lines[k].markers.%s = value' % name)
    validate_active()
    if not _active.hold in ['on','yes']:
        _active.line_list.data = [] # clear it out
        _active.image_list.data = [] # clear it out
    for i in lines:
        _active.line_list.append(i)
    _active.update()                
    return _active

def markers(visible=None):
    pass

#-------------------------------------------------------------------#
#--------------------------- image ---------------------------------#
#-------------------------------------------------------------------#

def image(img,x=None,y=None,colormap = 'grey',scale='no'):
    """Colormap should really default to the current colormap..."""
    # check for hold here    
    validate_active()
    image = wxplt.image_object(img,x,y,colormap=colormap,scale=scale)    
    if not _active.hold in ['on','yes']:
        _active.line_list.data = [] # clear it out
        _active.image_list.data = [] # clear it out
        _active.image_list.append(image)
        try:
            axis('equal')
        except AttributeError:
            # cluge to handle case where ticks didn't exist when
            # calling axis()
            _active.client.layout_all()
            axis('equal')
    else:
        _active.image_list.append(image)
        _active.update()                
    return _active

def imagesc(img,x=None,y=None,colormap = 'grey'):
    image(img,x,y,colormap,scale='yes')
    
#matlab equivalence
xlabel = xtitle
ylabel = ytitle     

def speed_test():
    p = plot([1,2,3],'r:o')
    s1 = (200,200)
    s2 = (400,400)
    p.SetSize(s1)
    for i in range(20):
        if p.GetSizeTuple()[0] == 200: 
            p.SetSize(s2)
        else: 
            p.SetSize(s1)



# Test interfaces to fortran blas.
#
# The tests are more of interface than they are of the underlying blas.
# Only very small matrices checked -- N=3 or so.
#
# These test really need to be checked on 64 bit architectures.
# What does complex32 become on such machines? complex64 I'll bet.
# If so, I think we are OK.
# Check when we have a machine to check on.
#
# !! Complex calculations really aren't checked that carefully.
# !! Only real valued complex numbers are used in tests.
#
# !! Uses matrixmultiply to check against blas.  If matrixmultiply is
# !! ever !replaced! by a blas call, we'll need to fill in a simple 
# !! matrix multiply here to ensure integrity of tests.

from Numeric import *
from scipy_base.fastumath import *

import sys
from scipy_test.testing import *
set_package_path()
from linalg import fblas
del sys.path[0]

import unittest

#decimal accuracy to require between Python and LAPACK/BLAS calculations
accuracy = 5

##################################################
### Test blas ?axpy

class base_axpy(unittest.TestCase):
    def check_default_a(self):
        x = arange(3.,typecode=self.typecode)
        y = arange(3.,typecode=x.typecode())
        real_y = x*1.+y
        self.blas_func(x,y)        
        assert_array_equal(real_y,y)
    def check_simple(self):
        x = arange(3.,typecode=self.typecode)
        y = arange(3.,typecode=x.typecode())
        real_y = x*3.+y
        self.blas_func(x,y,a=3.)        
        assert_array_equal(real_y,y)
    def check_x_stride(self):
        x = arange(6.,typecode=self.typecode)
        y = zeros(3,x.typecode())
        y = arange(3.,typecode=x.typecode())
        real_y = x[::2]*3.+y
        self.blas_func(x,y,a=3.,n=3,incx=2)        
        assert_array_equal(real_y,y)
    def check_y_stride(self):
        x = arange(3.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        real_y = x*3.+y[::2]
        self.blas_func(x,y,a=3.,n=3,incy=2)
        assert_array_equal(real_y,y[::2])
    def check_x_and_y_stride(self):
        x = arange(12.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        real_y = x[::4]*3.+y[::2]
        self.blas_func(x,y,a=3.,n=3,incx=4,incy=2)
        assert_array_equal(real_y,y[::2])
    def check_x_bad_size(self):
        x = arange(12.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        try:
            self.blas_func(x,y,n=4,incx=5)
        except: # what kind of error should be caught?
            return
        # should catch error and never get here                        
        assert(0)    
    def check_y_bad_size(self):
        x = arange(12.,typecode=Complex32)
        y = zeros(6,x.typecode())
        try:
            self.blas_func(x,y,n=3,incy=5)
        except: # what kind of error should be caught?
            return
        # should catch error and never get here                        
        assert(0)                

try:
    class test_saxpy(base_axpy):
        blas_func = fblas.saxpy
        typecode = Float32
except AttributeError:
    class test_saxpy: pass
class test_daxpy(base_axpy):
    blas_func = fblas.daxpy
    typecode = Float
try:
    class test_caxpy(base_axpy):
        blas_func = fblas.caxpy
        typecode = Complex32
except AttributeError:
    class test_caxpy: pass
class test_zaxpy(base_axpy):
    blas_func = fblas.zaxpy
    typecode = Complex


##################################################
### Test blas ?scal

class base_scal(unittest.TestCase):
    def check_simple(self):
        x = arange(3.,typecode=self.typecode)
        real_x = x*3.
        self.blas_func(3.,x)        
        assert_array_equal(real_x,x)
    def check_x_stride(self):
        x = arange(6.,typecode=self.typecode)
        real_x = x.copy()
        real_x[::2] = x[::2]*array(3.,self.typecode)
        self.blas_func(3.,x,n=3,incx=2)        
        assert_array_equal(real_x,x)
    def check_x_bad_size(self):
        x = arange(12.,typecode=self.typecode)
        try:
            self.blas_func(2.,x,n=4,incx=5)
        except: # what kind of error should be caught?
            return
        # should catch error and never get here                        
        assert(0)    
try:
    class test_sscal(base_scal):
        blas_func = fblas.sscal
        typecode = Float32
except AttributeError:
    class test_sscal: pass
class test_dscal(base_scal):
    blas_func = fblas.dscal
    typecode = Float
try:
    class test_cscal(base_scal):
        blas_func = fblas.cscal
        typecode = Complex32
except AttributeError:
    class test_cscal: pass
class test_zscal(base_scal):
    blas_func = fblas.zscal
    typecode = Complex




##################################################
### Test blas ?copy

class base_copy(unittest.TestCase):
    def check_simple(self):
        x = arange(3.,typecode=self.typecode)
        y = zeros(shape(x),x.typecode())
        self.blas_func(x,y)
        assert_array_equal(x,y)
    def check_x_stride(self):
        x = arange(6.,typecode=self.typecode)
        y = zeros(3,x.typecode())
        self.blas_func(x,y,n=3,incx=2)
        assert_array_equal(x[::2],y)
    def check_y_stride(self):
        x = arange(3.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        self.blas_func(x,y,n=3,incy=2)
        assert_array_equal(x,y[::2])
    def check_x_and_y_stride(self):
        x = arange(12.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        self.blas_func(x,y,n=3,incx=4,incy=2)
        assert_array_equal(x[::4],y[::2])
    def check_x_bad_size(self):
        x = arange(12.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        try:
            self.blas_func(x,y,n=4,incx=5)
        except: # what kind of error should be caught?
            return
        # should catch error and never get here                        
        assert(0)    
    def check_y_bad_size(self):
        x = arange(12.,typecode=Complex32)
        y = zeros(6,x.typecode())
        try:
            self.blas_func(x,y,n=3,incy=5)
        except: # what kind of error should be caught?
            return
        # should catch error and never get here                        
        assert(0)                
    #def check_y_bad_type(self):
    ##   Hmmm. Should this work?  What should be the output.
    #    x = arange(3.,typecode=self.typecode)
    #    y = zeros(shape(x))
    #    self.blas_func(x,y)
    #    assert_array_equal(x,y)

try:
    class test_scopy(base_copy):
        blas_func = fblas.scopy
        typecode = Float32
except AttributeError:
    class test_scopy: pass
class test_dcopy(base_copy):
    blas_func = fblas.dcopy
    typecode = Float
try:
    class test_ccopy(base_copy):
        blas_func = fblas.ccopy
        typecode = Complex32
except AttributeError:
    class test_ccopy: pass
class test_zcopy(base_copy):
    blas_func = fblas.zcopy
    typecode = Complex                       


##################################################
### Test blas ?swap

class base_swap(unittest.TestCase):
    def check_simple(self):
        x = arange(3.,typecode=self.typecode)
        y = zeros(shape(x),x.typecode())
        desired_x = y.copy()
        desired_y = x.copy()
        self.blas_func(x,y)
        assert_array_equal(desired_x,x)
        assert_array_equal(desired_y,y)
    def check_x_stride(self):
        x = arange(6.,typecode=self.typecode)
        y = zeros(3,x.typecode())
        desired_x = y.copy()
        desired_y = x.copy()[::2]
        self.blas_func(x,y,n=3,incx=2)
        assert_array_equal(desired_x,x[::2])
        assert_array_equal(desired_y,y)
    def check_y_stride(self):
        x = arange(3.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        desired_x = y.copy()[::2]
        desired_y = x.copy()
        self.blas_func(x,y,n=3,incy=2)
        assert_array_equal(desired_x,x)
        assert_array_equal(desired_y,y[::2])
    
    def check_x_and_y_stride(self):
        x = arange(12.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        desired_x = y.copy()[::2]
        desired_y = x.copy()[::4]
        self.blas_func(x,y,n=3,incx=4,incy=2)
        assert_array_equal(desired_x,x[::4])
        assert_array_equal(desired_y,y[::2])
    def check_x_bad_size(self):
        x = arange(12.,typecode=self.typecode)
        y = zeros(6,x.typecode())
        try:
            self.blas_func(x,y,n=4,incx=5)
        except: # what kind of error should be caught?
            return
        # should catch error and never get here                        
        assert(0)    
    def check_y_bad_size(self):
        x = arange(12.,typecode=Complex32)
        y = zeros(6,x.typecode())
        try:
            self.blas_func(x,y,n=3,incy=5)
        except: # what kind of error should be caught?
            return
        # should catch error and never get here                        
        assert(0)                

try:
    class test_sswap(base_swap):
        blas_func = fblas.sswap
        typecode = Float32
except AttributeError:
    class test_sswap: pass
class test_dswap(base_swap):
    blas_func = fblas.dswap
    typecode = Float
try:
    class test_cswap(base_swap):
        blas_func = fblas.cswap
        typecode = Complex32
except AttributeError:
    class test_cswap: pass
class test_zswap(base_swap):
    blas_func = fblas.zswap
    typecode = Complex                       

##################################################
### Test blas ?gemv
### This will be a mess to test all cases.

class base_gemv(unittest.TestCase):
    def get_data(self,x_stride=1,y_stride=1):
        mult = array(1, typecode = self.typecode)            
        if self.typecode in ['F', 'D']:
            mult = array(1+1j, typecode = self.typecode)
        from RandomArray import normal
        alpha = array(1., typecode = self.typecode) * mult
        beta = array(1.,typecode = self.typecode) * mult
        a = normal(0.,1.,(3,3)).astype(self.typecode) * mult
        x = arange(shape(a)[0]*x_stride,typecode=self.typecode) * mult
        y = arange(shape(a)[1]*y_stride,typecode=self.typecode) * mult
        return alpha,beta,a,x,y
    def check_simple(self):
        alpha,beta,a,x,y = self.get_data()
        desired_y = alpha*matrixmultiply(a,x)+beta*y
        y = self.blas_func(alpha,a,x,beta,y)
        assert(allclose(desired_y,y))
    def check_default_beta_y(self):
        alpha,beta,a,x,y = self.get_data()
        desired_y = matrixmultiply(a,x)
        y = self.blas_func(1,a,x)
        assert(allclose(desired_y,y))
    def check_simple_transpose(self):
        alpha,beta,a,x,y = self.get_data()
        desired_y = alpha*matrixmultiply(transpose(a),x)+beta*y
        y = self.blas_func(alpha,a,x,beta,y,trans=1)
        assert(allclose(desired_y,y))
    def check_simple_transpose_conj(self):
        alpha,beta,a,x,y = self.get_data()
        desired_y = alpha*matrixmultiply(transpose(conjugate(a)),x)+beta*y
        y = self.blas_func(alpha,a,x,beta,y,trans=2)
        assert(allclose(desired_y,y))
    def check_x_stride(self):
        alpha,beta,a,x,y = self.get_data(x_stride=2)
        desired_y = alpha*matrixmultiply(a,x[::2])+beta*y
        y = self.blas_func(alpha,a,x,beta,y,incx=2)
        assert(allclose(desired_y,y))
    def check_x_stride_transpose(self):
        alpha,beta,a,x,y = self.get_data(x_stride=2)
        desired_y = alpha*matrixmultiply(transpose(a),x[::2])+beta*y
        y = self.blas_func(alpha,a,x,beta,y,trans=1,incx=2)
        assert(allclose(desired_y,y))
    def check_x_stride_assert(self):
        # What is the use of this test?
        alpha,beta,a,x,y = self.get_data(x_stride=2)
        try:
            y = self.blas_func(1,a,x,1,y,trans=0,incx=3)
            assert(0)
        except:
            pass                    
        try:
            y = self.blas_func(1,a,x,1,y,trans=1,incx=3)
            assert(0)
        except:
            pass                    
    def check_y_stride(self):
        alpha,beta,a,x,y = self.get_data(y_stride=2)
        desired_y = y.copy()
        desired_y[::2] = alpha*matrixmultiply(a,x)+beta*y[::2]
        y = self.blas_func(alpha,a,x,beta,y,incy=2)
        assert(allclose(desired_y,y))
    def check_y_stride_transpose(self):
        alpha,beta,a,x,y = self.get_data(y_stride=2)
        desired_y = y.copy()        
        desired_y[::2] = alpha*matrixmultiply(transpose(a),x)+beta*y[::2]
        y = self.blas_func(alpha,a,x,beta,y,trans=1,incy=2)
        assert(allclose(desired_y,y))
    def check_y_stride_assert(self):
        # What is the use of this test?
        alpha,beta,a,x,y = self.get_data(y_stride=2)
        try:
            y = self.blas_func(1,a,x,1,y,trans=0,incy=3)
            assert(0)            
        except:
            pass
        try:
            y = self.blas_func(1,a,x,1,y,trans=1,incy=3)
            assert(0)            
        except:
            pass

try:
    class test_sgemv(base_gemv):
        blas_func = fblas.sgemv
        typecode = Float32
except AttributeError:
    class test_sgemv: pass
class test_dgemv(base_gemv):
    blas_func = fblas.dgemv
    typecode = Float
try:
    class test_cgemv(base_gemv):
        blas_func = fblas.cgemv
        typecode = Complex32
except AttributeError:
    class test_cgemv: pass
class test_zgemv(base_gemv):
    blas_func = fblas.zgemv
    typecode = Complex                       

"""
##################################################
### Test blas ?ger
### This will be a mess to test all cases.

class base_ger(unittest.TestCase):
    def get_data(self,x_stride=1,y_stride=1):
        from RandomArray import normal
        alpha = array(1., typecode = self.typecode)
        a = normal(0.,1.,(3,3)).astype(self.typecode)
        x = arange(shape(a)[0]*x_stride,typecode=self.typecode)
        y = arange(shape(a)[1]*y_stride,typecode=self.typecode)
        return alpha,a,x,y
    def check_simple(self):
        alpha,a,x,y = self.get_data()
        # tranpose takes care of Fortran vs. C(and Python) memory layout
        desired_a = alpha*transpose(x[:,NewAxis]*y) + a
        self.blas_func(x,y,a)
        assert(allclose(desired_a,a))
    def check_x_stride(self):
        alpha,a,x,y = self.get_data(x_stride=2)
        desired_a = alpha*transpose(x[::2,NewAxis]*y) + a
        self.blas_func(x,y,a,incx=2)
        assert(allclose(desired_a,a))
    def check_x_stride_assert(self):
        alpha,a,x,y = self.get_data(x_stride=2)
        try:
            self.blas_func(x,y,a,incx=3)
            assert(0)
        except:
            pass                    
    def check_y_stride(self):
        alpha,a,x,y = self.get_data(y_stride=2)
        desired_a = alpha*transpose(x[:,NewAxis]*y[::2]) + a
        self.blas_func(x,y,a,incy=2)
        assert(allclose(desired_a,a))

    def check_y_stride_assert(self):
        alpha,a,x,y = self.get_data(y_stride=2)
        try:
            self.blas_func(a,x,y,incy=3)
            assert(0)            
        except:
            pass

class test_sger(base_ger):
    blas_func = fblas.sger
    typecode = Float32
class test_dger(base_ger):
    blas_func = fblas.dger
    typecode = Float
"""
##################################################
### Test blas ?gerc
### This will be a mess to test all cases.

"""
class base_ger_complex(base_ger):
    def get_data(self,x_stride=1,y_stride=1):
        from RandomArray import normal
        alpha = array(1+1j, typecode = self.typecode)
        a = normal(0.,1.,(3,3)).astype(self.typecode)
        a = a + normal(0.,1.,(3,3)) * array(1j, typecode = self.typecode)
        x = normal(0.,1.,shape(a)[0]*x_stride).astype(self.typecode)
        x = x + x * array(1j, typecode = self.typecode)
        y = normal(0.,1.,shape(a)[1]*y_stride).astype(self.typecode)
        y = y + y * array(1j, typecode = self.typecode)
        return alpha,a,x,y
    def check_simple(self):
        alpha,a,x,y = self.get_data()
        # tranpose takes care of Fortran vs. C(and Python) memory layout
        a = a * array(0.,typecode = self.typecode)
        #desired_a = alpha*transpose(x[:,NewAxis]*self.transform(y)) + a
        desired_a = alpha*transpose(x[:,NewAxis]*y) + a
        #self.blas_func(x,y,a,alpha = alpha)
        fblas.cgeru(x,y,a,alpha = alpha)
        print x, y
        print desired_a.typecode(),desired_a
        print
        print a.typecode(),a
        assert(allclose(desired_a,a))

    #def check_x_stride(self):
    #    alpha,a,x,y = self.get_data(x_stride=2)
    #    desired_a = alpha*transpose(x[::2,NewAxis]*self.transform(y)) + a
    #    self.blas_func(x,y,a,incx=2)
    #    assert(allclose(desired_a,a))
    #def check_y_stride(self):
    #    alpha,a,x,y = self.get_data(y_stride=2)
    #    desired_a = alpha*transpose(x[:,NewAxis]*self.transform(y[::2])) + a
    #    self.blas_func(x,y,a,incy=2)
    #    assert(allclose(desired_a,a))

class test_cgeru(base_ger_complex):
    blas_func = fblas.cgeru
    typecode = Complex32
    def transform(self,x):
        return x
class test_zgeru(base_ger_complex):
    blas_func = fblas.zgeru
    typecode = Complex
    def transform(self,x):
        return x

class test_cgerc(base_ger_complex):
    blas_func = fblas.cgerc
    typecode = Complex32
    def transform(self,x):
        return conjugate(x)

class test_zgerc(base_ger_complex):
    blas_func = fblas.zgerc
    typecode = Complex
    def transform(self,x):
        return conjugate(x)
"""        

if __name__ == "__main__":
    ScipyTest('linalg.fblas').run()



import os
import sys

from cpuinfo import cpu
from fcompiler import FCompiler

class SunFCompiler(FCompiler):

    compiler_type = 'sun'
    version_pattern = r'(f90|f95): (Sun|Forte Developer 7|WorkShop 6 update \d+) Fortran 95 (?P<version>[^\s]+).*'

    executables = {
        'version_cmd'  : ["f90", "-V"],
        'compiler_f77' : ["f90"],
        'compiler_fix' : ["f90", "-fixed"],
        'compiler_f90' : ["f90"],
        'linker_so'    : ["f90","-Bdynamic","-G"],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"]
        }
    module_dir_switch = '-moddir='
    module_include_switch = '-M'
    pic_flags = ['-xcode=pic32']

    def get_flags_f77(self):
        ret = ["-ftrap=%none"]
        if (self.get_version() or '') >= '7':
            ret.append("-f77")
        else:
            ret.append("-fixed")
        return ret
    def get_opt(self):
        return ['-fast','-dalign']
    def get_arch(self):
        return ['-xtarget=generic']
    def get_libraries(self):
        opt = []
        opt.extend(['fsu','sunmath','mvec','f77compat'])
        return opt

if __name__ == '__main__':
    from scipy_distutils import log
    log.set_verbosity(2)
    from fcompiler import new_fcompiler
    compiler = new_fcompiler(compiler='sun')
    compiler.customize()
    print compiler.get_version()



#
# optimize - Optimization Tools
#

from info_optimize import __doc__

from optimize import *
from minpack import *
from zeros import *
from anneal import *
from lbfgsb import fmin_l_bfgs_b
from tnc import fmin_tnc
from cobyla import fmin_cobyla



"""
C/C++ integration
=================

        1. inline() -- a function for including C/C++ code within Python
        2. blitz()  -- a function for compiling Numeric expressions to C++
        3. ext_tools-- a module that helps construct C/C++ extension modules.
        4. accelerate -- a module that inline accelerates Python functions
"""
postpone_import = 1
standalone = 1




import sys
from scipy_test.testing import *

set_package_path()
# make sure that all yyy symbols are imported before the del statement:
from yyy import fun
del sys.path[0]

class test_fun(ScipyTestCase):
      def check_simple(self, level=1):
          assert fun()=='Hello from yyy.fun'
      #...

if __name__ == "__main__":
    ScipyTest('yyy').run()



# $Id: gistdemo3d.py,v 1.2 2003/09/30 23:43:47 travo Exp $
#  ---------------------------------------------------------------------
#
#  NAME:     gistdemo3d.py
#
#  PURPOSE:  Demonstrate 3D functions in pygist.
#
#  CHANGES:
#  04/22/03 llc This version is the original demo5 in gist3d with the
#               tests using the non-existent PR module commented out.
#               PR is part of the outdated pyPDB.  These tests read in
#               data files in Portable Data Base (PDB) or Silo format
#               and were likely not included in the original distribution
#               due to size.
#
#  ---------------------------------------------------------------------
# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from plwf import *
from pl3d import *
from movie import *
from slice3 import *
from yorick import *
from gist import *
from arrayfns import *
from RNG import * 

print "Type gistdemo3d.run() or gistdemo3d.run(i), i = 1, 2, or 3."
print "Partway, plots are written to Postscript file; see talk.ps."

window3 (hcp = "talk.ps", dump = 1)

palette ("gray.gp")

demo5_n = 20. * ones (3)

making_movie = 0

def demo5_light (i) :
   global making_movie
   if i >= 30 : return 0
   theta = pi / 4 + (i - 1) * 2 * pi/29
   light3 (sdir = array ( [cos(theta), .25, sin(theta)], Float))
   # without an explicit call to draw3, the light3 function would
   # cause no changes until Python paused for input from the keyboard,
   # since unlike the primitive plotting functions (plg, plf, plfp, ...)
   # the fma call made by the movie function will not trigger the
   # 3D display list
   # any movie frame display function which uses the 3D drawing
   # functions in pl3d.i will need to do this
   # the !making_movie flag supresses the fma in draw3 if this function
   # is called by movie (which issues its own fma), but allows it
   # otherwise 

   draw3 ( not making_movie )
   return 1

def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return

def run (*itest) :
   """run() or run(i)
     Run examples of use of pl3d.i, plwf.i, and slice3.i.  With
     argument I = 1, 2, or 3, run that particular demonstration.
     Read the source code to understand the details of how the
     various effects are obtained.

     run (1) demonstrates the various effects which can be obtained
     with the plwf (plot wire frame) function.
     run (2) demonstrates shading effects controlled by the light3
     function
     run (3) demonstrates the slice3, slice2, and pl3tree functions,
     as well as changing the orientation of the 3D object
   """
   global making_movie

   if len (itest) == 0 or itest [0] == 1 :
      set_draw3_ (0)
      x = span (-1, 1, 64, 64)
      y = transpose (x)
      z = (x + y) * exp (-6.*(x*x+y*y))
      limits_(square = 1)
      print "Plot wire frame: plwf (z, y, x)"
      orient3 ( )
      light3 ( )
      plwf (z, y, x)
      [xmin, xmax, ymin, ymax] = draw3(1) # not necessary interactively
      limits (xmin, xmax, ymin, ymax)
      plt("opaque wire mesh", .30, .42)
      paws ( )

      print 'plwf(z,y,x,shade=1,ecolor="red")'
      plwf(z,y,x,shade=1,ecolor="red")
      [xmin, xmax, ymin, ymax] = draw3(1) # not necessary interactively
      limits (xmin, xmax, ymin, ymax)
      paws()

      print "plwf(z,y,x,shade=1,edges=0)"
      plwf(z,y,x,shade=1,edges=0)
      [xmin, xmax, ymin, ymax] = draw3(1) # not necessary interactively
      limits (xmin, xmax, ymin, ymax)
      paws ( )

      print "light3 ( diffuse=.1, specular=1., sdir=array([0,0,-1]))"
      light3 ( diffuse=.1, specular=1., sdir=array([0,0,-1]))
      [xmin, xmax, ymin, ymax] = draw3(1)
      limits (xmin, xmax, ymin, ymax)
      paws ( )

      print "light3 ( diffuse=.5, specular=1., sdir=array([1,.5,1]))"
      light3 ( diffuse=.5, specular=1., sdir=array([1,.5,1]))
      [xmin, xmax, ymin, ymax] = draw3 (1)
      limits (xmin, xmax, ymin, ymax)
      paws ( )

      print "light3 ( ambient=.1,diffuse=.1,specular=1.,"
      print "   sdir=array([[0,0,-1],[1,.5,1]]),spower=array([4,2]))"
      light3 ( ambient=.1,diffuse=.1,specular=1.,
             sdir=array([[0,0,-1],[1,.5,1]]),spower=array([4,2]))
      [xmin, xmax, ymin, ymax] = draw3(1)
      limits (xmin, xmax, ymin, ymax)
      paws ( )

   if len (itest) == 0 or itest [0] == 2 :

      set_draw3_ (0)
      x = span (-1, 1, 64, 64)
      y = transpose (x)
      z = (x + y) * exp (-6.*(x*x+y*y))
      print "Default lighting:  light3()"
      orient3 ( )
      light3 ( )
      plwf (z,y,x,shade=1,edges=0)
      [xmin, xmax, ymin, ymax] = draw3 (1) # not necessary interactively
      limits (xmin, xmax, ymin, ymax)
      paws( )

      print "light3(diffuse=.2,specular=1)"
      light3(diffuse=.2,specular=1)
      limits_(square = 1)
      [xmin, xmax, ymin, ymax] = draw3(1) # not necessary interactively
      limits (xmin, xmax, ymin, ymax)
      paws()

      print "movie(demo5_light, lims = [xmin, xmax, ymin, ymax])"
      print "demo5_light calls:"
      print "light3 (sdir = array ( [cos(theta), .25, sin(theta)], Float))"
      making_movie = 1
      movie(demo5_light, lims = [xmin, xmax, ymin, ymax])
      making_movie = 0
      fma()
      demo5_light(1)
      paws()
      light3()

   if len (itest) == 0 or itest [0] == 3 :

      nx = demo5_n [0]
      ny = demo5_n [1]
      nz = demo5_n [2]
      xyz = zeros ( (3, nx, ny, nz), Float)
      xyz [0] = multiply.outer ( span (-1, 1, nx), ones ( (ny, nz), Float))
      xyz [1] = multiply.outer ( ones (nx, Float),
         multiply.outer ( span (-1, 1, ny), ones (nz, Float)))
      xyz [2] = multiply.outer ( ones ( (nx, ny), Float), span (-1, 1, nz))
      r = sqrt (xyz [0] ** 2 + xyz [1] **2 + xyz [2] **2)
      theta = arccos (xyz [2] / r)
      phi = arctan2 (xyz [1] , xyz [0] + logical_not (r))
      y32 = sin (theta) ** 2 * cos (theta) * cos (2 * phi)
      m3 = mesh3 (xyz, funcs = [r * (1. + y32)])
      del r, theta, phi, xyz, y32

      print "   test uses " + `(nx - 1) * (ny - 1) * (nz - 1)` + " cells"
      elapsed = [0., 0., 0.]
      elapsed = timer_ (elapsed)
      elapsed0 = elapsed

      [nv, xyzv, dum] = slice3 (m3, 1, None, None, value = .50)
          # (inner isosurface)
      [nw, xyzw, dum] = slice3 (m3, 1, None, None, value = 1.)
          # (outer isosurface)
      pxy = plane3 ( array ([0, 0, 1], Float ), zeros (3, Float))
      pyz = plane3 ( array ([1, 0, 0], Float ), zeros (3, Float))
      [np, xyzp, vp] = slice3 (m3, pyz, None, None, 1)
          # (pseudo-colored slice)
      [np, xyzp, vp] = slice2 (pxy, np, xyzp, vp)
          # (cut slice in half)
      [nv, xyzv, d1, nvb, xyzvb, d2] = \
          slice2x (pxy, nv, xyzv, None)
      [nv, xyzv, d1] = \
          slice2 (- pyz, nv, xyzv, None)
          # (...halve one of those halves)
      [nw, xyzw, d1, nwb, xyzwb, d2] = \
          slice2x ( pxy , nw, xyzw, None)
          # (split outer in halves)
      [nw, xyzw, d1] = \
          slice2 (- pyz, nw, xyzw, None)

      elapsed = timer_ (elapsed)
      timer_print ("slicing time", elapsed - elapsed0)

      fma ()
      print 'Generate palette for pl3tree:  split_palette ("earth.gp")'
      split_palette ("earth.gp")
      print "gnomon -- turn on gnomon"
      gnomon (1)

      print "pl3tree with 1 slicing plane, 2 isosurfaces"
      clear3 ()
      # Make sure we don't draw till ready
      set_draw3_ (0)
      pl3tree (np, xyzp, vp, pyz)
      pl3tree (nvb, xyzvb)
      pl3tree (nwb, xyzwb)
      pl3tree (nv, xyzv)
      pl3tree (nw, xyzw)
      orient3 ()
      light3 (diffuse = .2, specular = 1)
      limits ()
      limits (square=1)
      demo5_light (1)
      paws ()
      hcp ()

      print "spin3 animated rotation, use rot3 or orient3 for one frame"
      # don't want limits to autoscale during animation
      lims = limits ( )
      spin3 ()
      limits ( ) # back to autoscaling
      demo5_light (1)
      paws ()

      light3 ()
      gnomon (0)
      limits (square = 1)
      palette ("gray.gp")

#  .. PR is not available.  Comment out this section.
#   if len (itest) == 0 or itest [0] == 4 :
#      from PR import *
#      f = PR ('./bills_plot')
#      n_nodes = f.NumNodes
#      n_z = f.NodesOnZones
#      x = f.XNodeCoords
#      y = f.YNodeCoords
#      z = f.ZNodeCoords
#      c = f.ZNodeVelocity
#      n_zones = f.NumZones
#      # Put vertices in right order for Gist
#      n_z = transpose (
#         take (transpose (n_z), array ( [0, 4, 3, 7, 1, 5, 2, 6])))
#      m3 = mesh3 (x, y, z, funcs = [c], verts = n_z ) # [0:10])
#      [nv, xyzv, cv] = slice3 (m3, 1, None, None, 1, value = .9 * max (c) )
#      pyz = plane3 ( array ([1, 0, 0], Float ), zeros (3, Float))
#      pxz = plane3 ( array ([0, 1, 0], Float ), zeros (3, Float))
#
#      # draw a colored plane first
#      fma ()
#      clear3 ()
#      # Make sure we don't draw till ready
#      set_draw3_ (0)
#      [np, xyzp, vp] = slice3 (m3, pyz, None, None, 1)
#      pl3tree (np, xyzp, vp, pyz, split = 0)
#      palette ("rainbow.gp")
#      orient3 ()
#      demo5_light (1)
#      paws ()
#      
#
#     [nv, xyzv, d1] = \
#         slice2 (- pyz, nv, xyzv, None)
#      [nw, xyzw, cw] = slice3 (m3, 1, None, None, 1, value = .9 * min (c) )
#     [nw, xyzw, d1] = \
#         slice2 (- pyz, nw, xyzw, None)
#      [nvi, xyzvi, cvi] = slice3 (m3, 1, None, None, 1, value = .5 * min (c) )
#      [nvi, xyzvi, cvi] = \
#          slice2 (- pyz, nvi, xyzvi, cvi)
#      [nvj, xyzvj, cvj] = slice3 (m3, 1, None, None, 1, value = .5 * max (c) )
#      [nvj, xyzvj, cvj] = \
#          slice2 (- pyz, nvj, xyzvj, cvj)
#
#      fma ()
#      print "gnomon -- turn on gnomon"
#      gnomon (1)
#      clear3 ()
#      # Make sure we don't draw till ready
#      set_draw3_ (0)
#      pl3tree (nv, xyzv) # , cv)
#      pl3tree (nw, xyzw) # , cw)
#      pl3tree (nvi, xyzvi) # , cvi)
#      pl3tree (nvj, xyzvj) # , cvi)
#      orient3 ()
#      light3 (ambient = 0, diffuse = .5, specular = 1, sdir = [0, 0, -1])
#      limits (square=1)
#      palette ("gray.gp")
#      demo5_light (1)
#      paws ()
#      
#      print "spin3 animated rotation, use rot3 or orient3 for one frame"
#      # don't want limits to autoscale during animation
#      spin3 ()
#      limits ( ) # back to autoscaling
#      demo5_light (1)
#      paws ()
#
#      light3 ()
#      gnomon (0)
#      palette ("gray.gp")
#
#      draw3 ( 1 )
#      paws ()
#      clear3 ()
#      del nv, xyzv, cv, nw, xyzw, cw, nvi, xyzvi, cvi, nvj, xyzvj, cvj
#      # Make sure we don't draw till ready
#      set_draw3_ (0)
#      for i in range (8) :
#         [nv, xyzv, cv] = slice3 (m3, 1, None, None, 1, value = .9 * min (c) +
#             i * (.9 * max (c) - .9 * min (c)) / 8.)
#         [nv, xyzv, d1] = \
#             slice2 (pxz, nv, xyzv, None)
#         pl3tree (nv, xyzv)
#      orient3 ()
#      light3 (ambient = 0, diffuse = .5, specular = 1, sdir = [0, 0, -1])
#      limits (square=1)
#      palette ("heat.gp")
#      demo5_light (1)
#      paws ()
#      spin3 ()
#      limits ( ) # back to autoscaling
#      demo5_light (1)
#      paws ()
#      demo5_light (1)
#      paws ()
#
#   if len (itest) == 0 or itest [0] == 5 :
#      # Try bert's data
#      from PR import PR
#      f = PR ('./berts_plot')
#      nums = array ( [63, 63, 49], Int)
#      dxs = array ( [2.5, 2.5, 10.], Float )
#      x0s = array ( [-80., -80., 0.0], Float )
#      c = f.c
#
#      m3 = mesh3 (nums, dxs, x0s, funcs = [transpose (c)])
#      [nv, xyzv, dum] = slice3 (m3, 1, None, None, value = 6.5)
#      fma ()
#      clear3 ()
#      print "gnomon -- turn on gnomon"
#      gnomon (1)
#      # Make sure we don't draw till ready
#      set_draw3_ (0)
#      palette ("rainbow.gp")
#      pl3tree (nv, xyzv)
#      orient3 ()
#      light3 (diffuse = .2, specular = 1)
#      limits (square=1)
#      demo5_light (1)
#      paws ()
#      spin3 ()
#      demo5_light (1)
#      paws ()
#   if len (itest) == 0 or itest [0] == 6 :
#      # Try Bill's irregular mesh
#      from PR import PR
#      f = PR ("ball.s0001")
#      ZLss = f.ZLstruct_shapesize
#      ZLsc = f.ZLstruct_shapecnt
#      ZLsn = f.ZLstruct_nodelist
#      x = f.sap_mesh_coord0
#      y = f.sap_mesh_coord1
#      z = f.sap_mesh_coord2
#      c = f.W_vel_data
#      # Now we need to convert this information to avs-style data
#      istart = 0 # beginning index into ZLstruct_nodelist
#      NodeError = "NodeError"
#      ntet = 0
#      nhex = 0
#      npyr = 0
#      nprism = 0
#      nz_tet = []
#      nz_hex = []
#      nz_pyr = []
#      nz_prism = []
#      for i in range (4) :
#         if ZLss [i] == 4 : # TETRAHEDRON
#            nz_tet = reshape (ZLsn [istart: istart + ZLss [i] * ZLsc [i]],
#                     (ZLsc [i], ZLss [i]))
#            ntet = ZLsc [i]
#            istart = istart + ZLss [i] * ZLsc [i]
#         elif ZLss[i] == 5 : # PYRAMID
#            nz_pyr = reshape (ZLsn [istart: istart + ZLss [i] * ZLsc [i]],
#                     (ZLsc [i], ZLss [i]))
#            npyr = ZLsc [i]
#            # Now reorder the points (bill has the apex last instead of first)
#            nz_pyr = transpose (
#               take (transpose (nz_pyr), array ( [4, 0, 1, 2, 3])))
#            istart = istart + ZLss [i] * ZLsc [i]
#         elif ZLss[i] == 6 : # PRISM
#            nz_prism = reshape (ZLsn [istart: istart + ZLss [i] * ZLsc [i]],
#                     (ZLsc [i], ZLss [i]))
#            nprism = ZLsc [i]
#            # now reorder the points (bill goes around a square face 
#            # instead of traversing the opposite sides in the same direction.
#            nz_prism = transpose (
#               take (transpose (nz_prism), array ( [0, 1, 3, 2, 4, 5])))
#            istart = istart + ZLss [i] * ZLsc [i]
#         elif ZLss[i] == 8 : # HEXAHEDRON
#            nz_hex = reshape (ZLsn [istart: istart + ZLss [i] * ZLsc [i]],
#                     (ZLsc [i], ZLss [i]))
#            # now reorder the points (bill goes around a square face 
#            # instead of traversing the opposite sides in the same direction.
#            nz_hex = transpose (
#               take (transpose (nz_hex), array ( [0, 1, 3, 2, 4, 5, 7, 6])))
#            nhex = ZLsc [i]
#            istart = istart + ZLss [i] * ZLsc [i]
#         else :
#            raise NodeError, `ZLss[i]` + "is an incorrect number of nodes."

#      m3 = mesh3 (x, y, z, funcs = [c], verts = [nz_tet, nz_pyr, nz_prism,
#         nz_hex])
#      [nv, xyzv, cv] = slice3 (m3, 1, None, None, 1, value = .9 * max (c) )
#      pyz = plane3 ( array ([1, 0, 0], Float ), zeros (3, Float))
#      pxz = plane3 ( array ([0, 1, 0], Float ), zeros (3, Float))
#
#      # draw a colored plane first
#      fma ()
#      clear3 ()
#      # Make sure we don't draw till ready
#      set_draw3_ (0)
#      [np, xyzp, vp] = slice3 (m3, pyz, None, None, 1)
#      pl3tree (np, xyzp, vp, pyz, split = 0)
#      palette ("rainbow.gp")
#      orient3 ()
#      limits (square=1)
#      demo5_light (1)
#      paws ()
#
#      [nw, xyzw, cw] = slice3 (m3, 1, None, None, 1, value = .9 * min (c) )
#      [nvi, xyzvi, cvi] = slice3 (m3, 1, None, None, 1, value = .1 * min (c) )
#      [nvi, xyzvi, cvi] = \
#          slice2 (- pyz, nvi, xyzvi, cvi)
#      [nvj, xyzvj, cvj] = slice3 (m3, 1, None, None, 1, value = .1 * max (c) )
#      [nvj, xyzvj, cvj] = \
#          slice2 (- pyz, nvj, xyzvj, cvj)
#      [nvii, xyzvii, cvii] = slice3 (m3, 1, None, None, 1,
#         value = 1.e-12 * min (c) )
#      [nvii, xyzvii, cvii] = \
#          slice2 (- pyz, nvii, xyzvii, cvii)
#      [nvjj, xyzvjj, cvjj] = slice3 (m3, 1, None, None, 1,
#         value = 1.e-12 * max (c) )
#      [nvjj, xyzvjj, cvjj] = \
#          slice2 (- pyz, nvjj, xyzvjj, cvjj)
#
#      fma ()
#      print "gnomon -- turn on gnomon"
#      gnomon (1)
#      clear3 ()
#      # Make sure we don't draw till ready
#      set_draw3_ (0)
#      pl3tree (nv, xyzv) # , cv)
#      pl3tree (nw, xyzw) # , cw)
#      pl3tree (nvi, xyzvi) # , cvi)
#      pl3tree (nvj, xyzvj) # , cvj)
#      pl3tree (nvii, xyzvii) # , cvii)
#      pl3tree (nvjj, xyzvjj) # , cvjj)
#      orient3 ()
#      light3 (ambient = 0, diffuse = .5, specular = 1, sdir = [0, 0, -1])
#      limits (square=1)
#      palette ("gray.gp")
#      demo5_light (1)
#      paws ()
#      palette ("heat.gp")
#      paws ()

   if len (itest) == 0 or itest [0] == 7 :

      print "Test plwf on the sombrero function"
      # compute sombrero function
      x = arange (-20, 21, typecode = Float)
      y = arange (-20, 21, typecode = Float)
      z = zeros ( (41, 41), Float)
      r = sqrt (add.outer ( x ** 2, y **2)) + 1e-6
      z = sin (r) / r
      fma ()
      clear3 ()
      gnomon (0)
      # Make sure we don't draw till ready
      set_draw3_ (0)
      palette ("rainbow.gp")
      limits (square=1)
      orient3 ()
      light3 ()
      plwf (z, fill = z, ecolor = "black")
      [xmin, xmax, ymin, ymax] = draw3 (1)
      limits (xmin, xmax, ymin, ymax)
      paws ()

      print "Try smooth contours, log mode:"
      print 'plzcont (nv, xyzv, contours = 20, scale = "normal")'
      [nv, xyzv, dum] = slice3mesh (x, y, z)
      zmult = max (max (abs (x)), max (abs (y)))
      plzcont (nv, xyzv, contours = 20, scale = "normal")
      [xmin, xmax, ymin, ymax] = draw3 (1)
      limits (xmin, xmax, ymin, ymax)
      paws ()

      print 'plzcont (nv, xyzv, contours = 20, scale = "lin", edges=1)'
      plzcont (nv, xyzv, contours = 20, scale = "lin", edges=1)
      [xmin, xmax, ymin, ymax] = draw3 (1)
      limits (xmin, xmax, ymin, ymax)
      paws ()

      print 'plwf (z, fill = z, shade = 1, ecolor = "black")'
      plwf (z, fill = z, shade = 1, ecolor = "black")
      [xmin, xmax, ymin, ymax] = draw3 (1)
      limits (xmin, xmax, ymin, ymax)
      paws ()

      print "plwf (z, fill = z, shade = 1, edges = 0)"
      plwf (z, fill = z, shade = 1, edges = 0)
      [xmin, xmax, ymin, ymax] = draw3 (1)
      limits (xmin, xmax, ymin, ymax)
      paws ()

      print "light3(diffuse=.2,specular=1)"
      print "demo5_light calls:"
      print "light3 (sdir = array ( [cos(theta), .25, sin(theta)], Float))"
      light3(diffuse=.2,specular=1)
      making_movie = 1
      movie(demo5_light, lims = [xmin, xmax, ymin, ymax])
      making_movie = 0
      fma()
      demo5_light(1)
      paws ()

      print "plwf (z, fill = None, shade = 1, edges = 0)"
      plwf (z, fill = None, shade = 1, edges = 0)
      [xmin, xmax, ymin, ymax] = draw3 (1)
      palette("gray.gp")
      limits (xmin, xmax, ymin, ymax)
      paws ()


   if len (itest) == 0 or itest [0] == 8 :

      print "Test pl3surf on the sombrero function"
      # compute sombrero function
      nc1 = 100
      nv1 = nc1 + 1
      br = - (nc1 / 2)
      tr = nc1 / 2 + 1
      x = arange (br, tr, typecode = Float) * 40. / nc1
      y = arange (br, tr, typecode = Float) * 40. / nc1
      z = zeros ( (nv1, nv1), Float)
      r = sqrt (add.outer ( x ** 2, y **2)) + 1e-6
      z = sin (r) / r
      # In order to use pl3surf, we need to construct a mesh
      # using mesh3. The way I am going to do that is to define
      # a function on the 3d mesh so that the sombrero function
      # is its 0-isosurface.
      z0 = min (ravel (z))
      z0 = z0 - .05 * abs (z0)
      maxz = max (ravel (z))
      maxz = maxz + .05 * abs (maxz)
      zmult = max (max (abs (x)), max (abs (y)))
      dz = (maxz - z0)
      nxnynz = array ( [nc1, nc1, 1], Int)
      dxdydz = array ( [1.0, 1.0, zmult*dz], Float )
      x0y0z0 = array ( [float (br), float (br), z0*zmult], Float )
      meshf = zeros ( (nv1, nv1, 2), Float )
      meshf [:, :, 0] = zmult*z - (x0y0z0 [2])
      meshf [:, :, 1] = zmult*z - (x0y0z0 [2] + dxdydz [2])

      m3 = mesh3 (nxnynz, dxdydz, x0y0z0, funcs = [meshf])
      fma ()
      # Make sure we don't draw till ready
      set_draw3_ (0)
      pldefault(edges=0)
      [nv, xyzv, col] = slice3 (m3, 1, None, None, value = 0.)
      orient3 ()
      pl3surf (nv, xyzv)
      lim = draw3 (1)
      limits (lim [0], lim [1], 1.5*lim [2], 1.5*lim [3])
      palette ("gray.gp")
      paws ()

      print "Try new slicing function (slice3mesh) to get color graph"
      [nv, xyzv, col] = slice3mesh (nxnynz [0:2], dxdydz [0:2], x0y0z0 [0:2],
         zmult * z, color = zmult * z)
      pl3surf (nv, xyzv, values = col)
      lim = draw3 (1)
      dif = 0.5 * (lim [3] - lim [2])
      limits (lim [0], lim [1], lim [2] - dif, lim [3] + dif)

      palette ("rainbow.gp")
      paws ()

      print "Try plzcont -- see if smooth mode is possible"
      print "plzcont (nv, xyzv)"
      palette ("heat.gp")
      plzcont (nv, xyzv)
      draw3 (1)
      paws ()

      print "plzcont (nv, xyzv, contours = 20)"
      plzcont (nv, xyzv, contours = 20)
      draw3 (1)
      paws ()

      print 'plzcont (nv, xyzv, contours = 20, scale = "log")'
      plzcont (nv, xyzv, contours = 20, scale = "log")
      draw3(1)
      paws ()

      print 'plzcont (nv, xyzv, contours = 20, scale = "normal")'
      plzcont (nv, xyzv, contours = 20, scale = "normal")
      draw3(1)
      paws ()

   if len (itest) == 0 or itest [0] == 9 :

      vsf = 0.
      c = 1
      s = 1000.
      kmax = 25
      lmax = 35

      # The following computations define an interesting 3d surface.

      xr = multiply.outer (
         arange (1, kmax + 1, typecode = Float), ones (lmax, Float))
      yr = multiply.outer (
         ones (kmax, Float), arange (1, lmax + 1, typecode = Float))
      zt = 5. + xr + .2 * random_sample (kmax, lmax)   # ranf (xr)
      rt = 100. + yr + .2 * random_sample (kmax, lmax)   # ranf (yr)
      z = s * (rt + zt)
      z = z + .02 * z * random_sample (kmax, lmax)   # ranf (z)
      ut = rt/sqrt (rt ** 2 + zt ** 2)
      vt = zt/sqrt (rt ** 2 + zt ** 2)
      ireg =  multiply.outer ( ones (kmax, Float), ones (lmax, Float))
      ireg [0:1, 0:lmax]=0
      ireg [0:kmax, 0:1]=0
      ireg [1:15, 7:12]=2
      ireg [1:15, 12:lmax]=3
      ireg [3:7, 3:7]=0
      freg=ireg + .2 * (1. - random_sample (kmax, lmax))  # ranf (ireg))
      freg=array (freg, Float)
      #rt [4:6, 4:6] = -1.e8
      z [3:10, 3:12] = z [3:10, 3:12] * .9
      z [5, 5] = z [5, 5] * .9
      z [17:22, 15:18] = z [17:22, 15:18] * 1.2
      z [16, 16] = z [16, 16] * 1.1
      orient3 ()
      print "plwf (freg, shade = 1, edges = 0)"
      plwf (freg, shade = 1, edges = 0)
      [xmin, xmax, ymin, ymax] = draw3 (1)
      limits (xmin, xmax, ymin, ymax)
      paws ()

      print "two slice3mesh and two pl3tree"
      nxny = array ( [kmax - 1, lmax - 1])
      x0y0 = array ( [0., 0.])
      dxdy = array ( [1., 1.])
      [nv, xyzv, col] = slice3mesh (nxny, dxdy, x0y0, freg)
      [nw, xyzw, col] = slice3mesh (nxny, dxdy, x0y0, freg + ut)
      pl3tree (nv, xyzv)
      pl3tree (nw, xyzw)
      draw3 (1)
      limits ( )
      paws ()

      print "light3 (ambient = 0, diffuse = .5, specular = 1, sdir = [0, 0, -1])"
      light3 (ambient = 0, diffuse = .5, specular = 1, sdir = [0, 0, -1])
      demo5_light (1)
      paws ()

      print "[nv, xyzv, col] = slice3mesh (nxny, dxdy, x0y0, freg, color = freg)"
      [nv, xyzv, col] = slice3mesh (nxny, dxdy, x0y0, freg, color = freg)
      pl3surf (nv, xyzv, values = col)
      draw3 (1)
      palette ("rainbow.gp")
      paws ()

      print 'palette ("rainbow.gp")'
      [nv, xyzv, col] = slice3mesh (nxny, dxdy, x0y0, freg, color = z)
      pl3surf (nv, xyzv, values = col)
      draw3 (1)
      paws ()

      print 'palette ("stern.gp")'
      palette ("stern.gp")
      paws ()

      print "[nv, xyzv, col] = slice3mesh (nxny, dxdy, x0y0, z, color = z)"
      [nv, xyzv, col] = slice3mesh (nxny, dxdy, x0y0, z, color = z)
      pl3surf (nv, xyzv, values = col)
      orient3(phi=0,theta=0)
      draw3 (1)
      paws ()

      print 'palette ("gray.gp")'
      print "light3 ( diffuse=.1, specular=1., sdir=array([0,0,-1]))"
      set_draw3_ (0)
      palette ("gray.gp")
      light3 ( diffuse=.1, specular=1., sdir=array([0,0,-1]))
      pl3surf (nv, xyzv)
      draw3 (1)
      paws ()

#     spin3 ()
#     paws ()

   hcp_finish ()



""" Modified version of build_ext that handles fortran source files.
"""

import os
import string
import sys
from glob import glob
from types import *

from distutils.dep_util import newer_group, newer
from distutils.command.build_ext import build_ext as old_build_ext

from scipy_distutils.command.build_clib import get_headers,get_directories
from scipy_distutils import misc_util, log
from scipy_distutils.misc_util import filter_sources, has_f_sources, \
     has_cxx_sources
from distutils.errors import DistutilsFileError


class build_ext (old_build_ext):

    description = "build C/C++/F extensions (compile/link to build directory)"

    user_options = old_build_ext.user_options + [
        ('fcompiler=', None,
         "specify the Fortran compiler type"),
        ]

    def initialize_options(self):
        old_build_ext.initialize_options(self)
        self.fcompiler = None
        return

    def finalize_options(self):
        old_build_ext.finalize_options(self)
        self.set_undefined_options('config_fc',
                                   ('fcompiler', 'fcompiler'))
        return

    def run(self):
        if not self.extensions:
            return

        # Make sure that extension sources are complete.
        for ext in self.extensions:
            if not misc_util.all_strings(ext.sources):
                raise TypeError,'Extension "%s" sources contains unresolved'\
                      ' items (call build_src before build_ext).' % (ext.name)

        if self.distribution.has_c_libraries():
            build_clib = self.get_finalized_command('build_clib')
            self.library_dirs.append(build_clib.build_clib)
        else:
            build_clib = None

        # Not including C libraries to the list of
        # extension libraries automatically to prevent
        # bogus linking commands. Extensions must
        # explicitly specify the C libraries that they use.

        # Determine if Fortran compiler is needed.
        if build_clib and build_clib.fcompiler is not None:
            need_f_compiler = 1
        else:
            need_f_compiler = 0
            for ext in self.extensions:
                if has_f_sources(ext.sources):
                    need_f_compiler = 1
                    break
                if getattr(ext,'language','c') in ['f77','f90']:
                    need_f_compiler = 1
                    break

        # Determine if C++ compiler is needed.
        need_cxx_compiler = 0
        for ext in self.extensions:
            if has_cxx_sources(ext.sources):
                need_cxx_compiler = 1
                break
            if getattr(ext,'language','c')=='c++':
                need_cxx_compiler = 1
                break

        from distutils.ccompiler import new_compiler
        self.compiler = new_compiler(compiler=self.compiler,
                                     verbose=self.verbose,
                                     dry_run=self.dry_run,
                                     force=self.force)
        self.compiler.customize(self.distribution,need_cxx=need_cxx_compiler)
        self.compiler.customize_cmd(self)
        self.compiler.show_customization()
 
        # Initialize Fortran/C++ compilers if needed.
        if need_f_compiler:
            from scipy_distutils.fcompiler import new_fcompiler
            self.fcompiler = new_fcompiler(compiler=self.fcompiler,
                                           verbose=self.verbose,
                                           dry_run=self.dry_run,
                                           force=self.force)
            self.fcompiler.customize(self.distribution)
            self.fcompiler.customize_cmd(self)
            self.fcompiler.show_customization()

        # Build extensions
        self.build_extensions()
        return

    def swig_sources(self, sources):
        # Do nothing. Swig sources have beed handled in build_src command.
        return sources

    def build_extension(self, ext):
        sources = ext.sources
        if sources is None or type(sources) not in (ListType, TupleType):
            raise DistutilsSetupError, \
                  ("in 'ext_modules' option (extension '%s'), " +
                   "'sources' must be present and must be " +
                   "a list of source filenames") % ext.name
        sources = list(sources)

        if not sources:
            return

        fullname = self.get_ext_fullname(ext.name)
        if self.inplace:
            modpath = string.split(fullname, '.')
            package = string.join(modpath[0:-1], '.')
            base = modpath[-1]

            build_py = self.get_finalized_command('build_py')
            package_dir = build_py.get_package_dir(package)
            ext_filename = os.path.join(package_dir,
                                        self.get_ext_filename(base))
        else:
            ext_filename = os.path.join(self.build_lib,
                                        self.get_ext_filename(fullname))
        depends = sources + ext.depends

        if not (self.force or newer_group(depends, ext_filename, 'newer')):
            log.debug("skipping '%s' extension (up-to-date)", ext.name)
            return
        else:
            log.info("building '%s' extension", ext.name)

        extra_args = ext.extra_compile_args or []
        macros = ext.define_macros[:]
        for undef in ext.undef_macros:
            macros.append((undef,))

        c_sources, cxx_sources, f_sources, fmodule_sources = \
                   filter_sources(ext.sources)
        if self.compiler.compiler_type=='msvc':
            if cxx_sources:
                # Needed to compile kiva.agg._agg extension.
                extra_args.append('/Zm1000')
            # this hack works around the msvc compiler attributes
            # problem, msvc uses its own convention :(
            c_sources += cxx_sources
            cxx_sources = []

        if sys.version[:3]>='2.3':
            kws = {'depends':ext.depends}
        else:
            kws = {}

        c_objects = []
        if c_sources:
            log.info("compiling C sources")
            c_objects = self.compiler.compile(c_sources,
                                              output_dir=self.build_temp,
                                              macros=macros,
                                              include_dirs=ext.include_dirs,
                                              debug=self.debug,
                                              extra_postargs=extra_args,
                                              **kws)
        if cxx_sources:
            log.info("compiling C++ sources")

            old_compiler = self.compiler.compiler_so[0]
            self.compiler.compiler_so[0] = self.compiler.compiler_cxx[0]

            c_objects += self.compiler.compile(cxx_sources,
                                              output_dir=self.build_temp,
                                              macros=macros,
                                              include_dirs=ext.include_dirs,
                                              debug=self.debug,
                                              extra_postargs=extra_args,
                                              **kws)
            self.compiler.compiler_so[0] = old_compiler

        check_for_f90_modules = not not fmodule_sources

        if f_sources or fmodule_sources:
            extra_postargs = []
            include_dirs = ext.include_dirs[:]
            module_dirs = ext.module_dirs[:]

            #if self.fcompiler.compiler_type=='ibm':
            macros = []

            if check_for_f90_modules:
                module_build_dir = os.path.join(\
                    self.build_temp,os.path.dirname(\
                    self.get_ext_filename(fullname)))

                self.mkpath(module_build_dir)
                if self.fcompiler.module_dir_switch is None:
                    existing_modules = glob('*.mod')
                extra_postargs += self.fcompiler.module_options(\
                    module_dirs,module_build_dir)

            f_objects = []
            if fmodule_sources:
                log.info("compiling Fortran 90 module sources")
                f_objects = self.fcompiler.compile(fmodule_sources,
                                                   output_dir=self.build_temp,
                                                   macros=macros,
                                                   include_dirs=include_dirs,
                                                   debug=self.debug,
                                                   extra_postargs=extra_postargs,
                                                   depends=ext.depends)

            if check_for_f90_modules \
                   and self.fcompiler.module_dir_switch is None:
                for f in glob('*.mod'):
                    if f in existing_modules:
                        continue
                    try:
                        self.move_file(f, module_build_dir)
                    except DistutilsFileError:  # already exists in destination
                        os.remove(f)
                        
            if f_sources:
                log.info("compiling Fortran sources")
                f_objects += self.fcompiler.compile(f_sources,
                                                    output_dir=self.build_temp,
                                                    macros=macros,
                                                    include_dirs=include_dirs,
                                                    debug=self.debug,
                                                    extra_postargs=extra_postargs,
                                                    depends=ext.depends)
        else:
            f_objects = []

        objects = c_objects + f_objects

        if ext.extra_objects:
            objects.extend(ext.extra_objects)
        extra_args = ext.extra_link_args or []

        try:
            old_linker_so_0 = self.compiler.linker_so[0]
        except:
            pass
        
        use_fortran_linker = getattr(ext,'language','c') in ['f77','f90']
        c_libraries = []
        c_library_dirs = []
        if use_fortran_linker or f_sources:
            use_fortran_linker = 1
        elif self.distribution.has_c_libraries():            
            build_clib = self.get_finalized_command('build_clib')
            f_libs = []
            for (lib_name, build_info) in build_clib.libraries:
                if has_f_sources(build_info.get('sources',[])):
                    f_libs.append(lib_name)
                if lib_name in ext.libraries:
                    # XXX: how to determine if c_libraries contain
                    # fortran compiled sources?
                    c_libraries.extend(build_info.get('libraries',[]))
                    c_library_dirs.extend(build_info.get('library_dirs',[]))
            for l in ext.libraries:
                if l in f_libs:
                    use_fortran_linker = 1
                    break

        # Always use system linker when using MSVC compiler.
        if self.compiler.compiler_type=='msvc' and use_fortran_linker:
            c_libraries.extend(self.fcompiler.libraries)
            c_library_dirs.extend(self.fcompiler.library_dirs)
            use_fortran_linker = 0

        if use_fortran_linker:
            if cxx_sources:
                # XXX: Which linker should be used, Fortran or C++?
                log.warn('mixing Fortran and C++ is untested')
            link = self.fcompiler.link_shared_object
            language = ext.language or self.fcompiler.detect_language(f_sources)
        else:
            link = self.compiler.link_shared_object
            if sys.version[:3]>='2.3':
                language = ext.language or self.compiler.detect_language(sources)
            else:
                language = ext.language
            if cxx_sources:
                self.compiler.linker_so[0] = self.compiler.compiler_cxx[0]

        if sys.version[:3]>='2.3':
            kws = {'target_lang':language}
        else:
            kws = {}

        link(objects, ext_filename,
             libraries=self.get_libraries(ext) + c_libraries,
             library_dirs=ext.library_dirs + c_library_dirs,
             runtime_library_dirs=ext.runtime_library_dirs,
             extra_postargs=extra_args,
             export_symbols=self.get_export_symbols(ext),
             debug=self.debug,
             build_temp=self.build_temp,**kws)

        try:
            self.compiler.linker_so[0] = old_linker_so_0
        except:
            pass

        return

    def get_source_files (self):
        self.check_extensions_list(self.extensions)
        filenames = []
        def visit_func(filenames,dirname,names):
            if os.path.basename(dirname) in ['CVS','.svn']:
                names[:] = []
                return
            for name in names:
                if name[-1] in "~#":
                    continue
                fullname = os.path.join(dirname,name)
                if os.path.isfile(fullname):
                    filenames.append(fullname)
        # Get sources and any include files in the same directory.
        for ext in self.extensions:
            sources = filter(lambda s:type(s) is StringType,ext.sources)
            filenames.extend(sources)
            filenames.extend(get_headers(get_directories(sources)))
            for d in ext.depends:
                if is_local_src_dir(d):
                    os.path.walk(d,visit_func,filenames)
                elif os.path.isfile(d):
                    filenames.append(d)
        return filenames

    def get_outputs (self):
        self.check_extensions_list(self.extensions)

        outputs = []
        for ext in self.extensions:
            if not ext.sources:
                continue
            fullname = self.get_ext_fullname(ext.name)
            outputs.append(os.path.join(self.build_lib,
                                        self.get_ext_filename(fullname)))
        return outputs

def is_local_src_dir(directory):
    """ Return true if directory is local directory.
    """
    abs_dir = os.path.abspath(directory)
    c = os.path.commonprefix([os.getcwd(),abs_dir])
    new_dir = abs_dir[len(c):].split(os.sep)
    if new_dir and not new_dir[0]:
        new_dir = new_dir[1:]
    if new_dir and new_dir[0]=='build':
        return 0
    new_dir = os.sep.join(new_dir)
    return os.path.isdir(new_dir)



"""
Scipy example module for developers

This is documentation of xxx.
Provides:
  xxx_foo_bar - also available in scipy name space
  foo_gun
  yyy.fun
"""

global_symbols = ['xxx_foo_bar']

postpone_import = 1



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
from MLab import rand
from surface import *
from graph3d import *
from mesh3d import *
def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return

GraphicsError = "GraphicsError"
import os
try:
   graphics = os.environ["PYGRAPH"]
except KeyError:
   graphics = "Gist"

if graphics [0:3] == "Nar" :
   import NarPlotter
elif graphics == "Gist" :
   import GistPlotter
else :
   raise GraphicsError , \
      graphics + " is an unknown graphics package. Check PYGRAPH " + \
         "environment variable."

if graphics [0:3] == "Nar" :
   print "This is a test of the Python interface to the Limeil Lab graphics"
   print "package, Narcisse. You  need Narcisse to be running. Fire it up by"
   print "doing setenv PORT_SERVEUR 0, typing  /dist/basis/Narcisse/bin/Narcisse &,"
   print "and then do another senetv PORT_SERVEUR to the port number which"
   print "appears in the Narcisse GUI."
else :
   print "This is a test of the Python interface to the Gist graphics package."

print "Invoke function demo () to run."

def demo () :
   vsf = 0.
   c = 1
   s = 1000.
   kmax = 25
   lmax = 35
   # The following computations define an interesting 3d surface.
   xr = multiply.outer (
      arange (1, kmax + 1, typecode = Float), ones (lmax, Float))
   yr = multiply.outer (
      ones (kmax, Float), arange (1, lmax + 1, typecode = Float))
   zt = 5. + xr + .2 * rand (kmax, lmax)   # ranf (xr)
   rt = 100. + yr + .2 * rand (kmax, lmax)   # ranf (yr)
   z = s * (rt + zt)
   z = z + .02 * z * rand (kmax, lmax)   # ranf (z)
   ut = rt / sqrt (rt ** 2 + zt ** 2)
   vt = zt / sqrt (rt ** 2 + zt ** 2)
   ireg =  multiply.outer ( ones (kmax), ones (lmax))
   ireg [0:1, 0:lmax] = 0
   ireg [0:kmax, 0:1] = 0
   ireg [1:15, 7:12] = 2
   ireg [1:15, 12:lmax] = 3
   ireg [3:7, 3:7] = 0
   freg = ireg + .2 * (1. - rand (kmax, lmax))  # ranf (ireg))
   freg = array (freg, Float)
   #rt [4:6, 4:6] = -1.e8
   z [3:10, 3:12] = z [3:10, 3:12] * .9
   z [5, 5] = z [5, 5] * .9
   z [17:22, 15:18] = z [17:22, 15:18] * 1.2
   z [16, 16] = z [16, 16] * 1.1
    
   # Sombrero function
   x = arange (-20, 21, typecode = Float)
   y = arange (-20, 21, typecode = Float)
   z = zeros ( (41, 41), Float)
   for i in range (0, 41):
       for j in range (0, 41):
           r = sqrt (x [i] ** 2 + y [j] ** 2) + 1e-6
           z [i, j] = sin (r) / r
   s1 = Surface (z = z, opt_3d = "s3", mask = "max")
   g1 = Graph3d ( s1 , titles = "Sombrero function",
                  y_factor = 2.0) #, phi = 30., theta = 45.)
   g1.plot ( )
   paws ( )
   s1.set (opt_3d = "w3")
   g1.plot ()
   paws ( )
   foo = zeros ( (kmax, lmax), Float)
   for k in range (kmax) :
      foo [k, 0:lmax] = log (arange (1, lmax + 1, typecode = Float))
      xxx = exp (k / 5.)
      for l in range (lmax) :
         foo [k, l] = xxx * foo [k, l]
   s1.new (x = xr, y = yr, z = foo, opt_3d = "w3")
   g1.change ( titles = "Exponential surface, z logarithmic", 
            # phi = 30., theta = 45.,
            axis_scales = ["lin", "lin", "log"],
            z_contours_scale = "log", y_factor = 1.0)
   g1.plot ( )
   paws ( )
   xr = array (xr)
   yr = array (yr)
   zz = xr ** 2 + yr ** 2
   g1.change ( titles = "Exponential surface, z linear", phi = 45.,
            axis_scales = ["lin", "lin", "lin"] )
   g1.plot ( )
   paws ( )
   s1.new (z = zz, opt_3d = "w3")
   g1.change ( z_contours_scale = "lin",
            titles = "Graph of xr**2 + yr**2", phi = 35.)
   g1.plot ( )
   paws ( )
   zs = zeros ( (50, 25), Float)
   xs = zeros ( (50, 25), Float)
   ys = zeros ( (50, 25), Float)
   ctheta = zeros (50, Float)
   stheta = zeros (50, Float)
   cphi = zeros (25, Float)
   sphi = zeros (25, Float)
   pi = 3.1415926535
   for i in range (50):
      ctheta [i] = cos (2. * i / 49. * pi)
      stheta [i] = sin (2. * i / 49. * pi)
   for i in range (25):
      cphi [i] = cos (pi * i / 24.)
      sphi [i] = sin (pi * i / 24.)
   for i in range (25):
      zs [0:50, i:i + 1] = cphi [i]
      for j in range (50):
         xs [j, i] = sphi [i] * ctheta [j]
         ys [j, i] = sphi [i] * stheta [j]
   hx = xs  / 1.6
   hy = ys / 1.6
   hz = zs / 1.6
   # Plot a sphere with a random distribution of colors
   s1.new (mask = "sort", opt_3d = "s4", x = hx, y = hy, z = zs,
           c = rand (50, 25)) # ranf (zs))
   g1.replace ( 1, s1 )
   g1.change_plot (titles = "Randomly colored sphere", send = 1)
   paws ()
   s1 = Surface (mask = "sort", opt_3d = "s4", x = hx, y = hy,
                 z = zs, c = rand (50, 25))
   ##m1 = Mesh3d (color_card = "random", opt_3d = "f4", mask = "min",
   ##             x = array ( [0.5, 1.2, 1.4, 1.05], Float),
   ##             y = array ( [1.2, 0.5, 1.4, 1.05], Float),
   ##             z = array ( [0., 0., 0., 1.2], Float),
   ##             c = array ( [0.4, 0.6, 0.6, 0.6], Float),
   ##             avs = 1, tet = [ 1, array ([[3,0,1,2]],Int) ])
   m1 = Slice (array ( [3, 3, 3, 3], Int),
               array ( [
                        [0.5, 1.2, 0.], [1.2, 0.5, 0.], [1.4, 1.4, 0.],
                        [0.5, 1.2, 0.], [1.4, 1.4, 0.], [1.05, 1.05, 1.2],
                        [0.5, 1.2, 0.], [1.05, 1.05, 1.2], [1.2, 0.5, 0.],
                        [1.2, 0.5, 0.], [1.05, 1.05, 1.2], [1.4, 1.4, 0.]
                     ] ), array ( [0.4, 0.5, 0.6, 0.7], Float))
   g1.new ([s1, m1], link = 1,
                   titles = "Randomly colored sphere and solid tetrahedron",
                   axis_limits = [[-0.62, 1.4],
                   [-0.62, 1.4], [-1.0, 1.5]],
                   send = 1, y_factor = 2.0) # phi = 55, theta = 30,
   g1.plot ( )
   #pl.set_color_card (8) #random
   #pl.set_3d_options ("f4") #flat 4d
   #pl.set_grid_type ("none") #don't redraw axes
   # Note to myself: When the mesh plotter is done, use it for the
   # following. Python should never call a plotter directly.
   #pl.plot_surface (array ( [0.5, 1.2, 1.4, 1.05], Float),
   #                 array ( [1.2, 0.5, 1.4, 1.05], Float),
   #                 array ( [0., 0., 0., 1.2], Float),
   #                 array ( [0.4, 0.6, 0.6, 0.6], Float),
   #                 array ( [3, 6, 9, 12, 1, 2, 3, 0, 2,
   #                          3, 0, 1, 3, 0, 1, 2], Int), 4)
   #pl.send_graph ()
   paws ()
   s1 = Surface ( mask = "sort", opt_3d = "s4", x = xs, y = ys,
                  z = zs + 2., c = zs)
   s2 = Surface ( mask = "sort", opt_3d = "wm", x = hx, y = hy, z = hz, c = zs)
   # g1.replace (1, s1)
   # g1.add ( s2 )
   # g1.change_plot ( link = 1 , titles = "Imploding Sphere in R3",
   #                  phi = 70.0, theta = 30.0,
   #                  axis_limits  =  [[-1., 1.], [-1., 1.], [-0.7, 3.0]])
   g1.new ( [s1, s2], link = 1 , titles = "Two Spheres in R3",
                      axis_limits  =  [[-1., 1.], [-1., 1.], [-0.7, 3.0]],
                      x_factor = 1.7)
   ##                 phi = 70.0, theta = 30.0,
   g1.plot ( )



# still needed
# tests for MingW32Compiler
# don't know how to test gcc_exists() and msvc_exists()...

import unittest
import os, sys, tempfile

from scipy_test.testing import *
set_package_path()
from weave import build_tools
restore_path()

def is_writable(val):
    return os.access(val,os.W_OK)
    
class test_configure_build_dir(unittest.TestCase):
    def check_default(self):
        " default behavior is to return current directory "
        d = build_tools.configure_build_dir()
        if is_writable('.'):
            assert(d == os.path.abspath('.'))
        assert(is_writable(d))
    def check_curdir(self):
        " make sure it handles relative values. "
        d = build_tools.configure_build_dir('.')
        if is_writable('.'):
            assert(d == os.path.abspath('.'))
        assert(is_writable(d))    
    def check_pardir(self):
        " make sure it handles relative values "        
        d = build_tools.configure_build_dir('..')
        if is_writable('..'):
            assert(d == os.path.abspath('..'))
        assert(is_writable(d))                
    def check_bad_path(self):
        " bad path should return same as default (and warn) "
        d = build_tools.configure_build_dir('_bad_path_')
        d2 = build_tools.configure_build_dir()
        assert(d == d2)
        assert(is_writable(d))

class test_configure_temp_dir(test_configure_build_dir):
    def check_default(self):
        " default behavior returns tempdir"
        # this'll fail if the temp directory isn't writable.
        d = build_tools.configure_temp_dir()
        assert(d == tempfile.gettempdir())
        assert(is_writable(d))

class test_configure_sys_argv(unittest.TestCase):
    def check_simple(self):
        build_dir = 'build_dir'
        temp_dir = 'temp_dir'
        compiler = 'compiler'
        pre_argv = sys.argv[:]
        build_tools.configure_sys_argv(compiler,temp_dir,build_dir)
        argv = sys.argv[:]
        bd = argv[argv.index('--build-lib')+1]
        assert(bd == build_dir)
        td = argv[argv.index('--build-temp')+1]
        assert(td == temp_dir)
        argv.index('--compiler='+compiler)
        build_tools.restore_sys_argv()
        assert(pre_argv == sys.argv[:])

if __name__ == "__main__":
    ScipyTest('weave.build_tools').run()



#!/usr/bin/env python
#  $Id: tkgist.py,v 1.1 2003/09/30 17:32:40 travo Exp $
#  ---------------------------------------------------------------------
#
#  NAME:  tkgist.py
#
#  CHANGES:
#  12/06/02 dhm Originated.
#  01/21/03 llc gistC should be gist.
#  
#  ---------------------------------------------------------------------
"""
Make gistC module compatible with _tkinter module
(The work is done when this module is imported.)

Need to ensure two things:
(1) pyg_idler gets called by tcl/tk as an "after idle" method
    - this allows gist to complete deferred actions
(2) pyg_pending gets called by tcl/tk whenever events arrive for
    a gist window
    - this is unnecessary under win32 because the OS automatically
      sends window messages to the gist winproc
"""

__version__ = "$Id: tkgist.py,v 1.1 2003/09/30 17:32:40 travo Exp $"

import sys
import Tkinter
import _tkinter
import gist

_root = None
_refresh = 1

# problem: how can you reset an "after idle" to make it into a true
#          idle event??

def do_pending(file, mask):
   global _root
   global _refresh
   print "python do_pending"
   if _root and _refresh:
      _refresh = 0
   gist.pyg_pending()

#def do_idle(*a):
def do_idle(dis, fd):
   global _root
   global _refresh
   print "python do_idle; dis=", dis, "fd=", fd
   gist.pyg_idler()
   _refresh = 1

   if _root:
      print _root
      if dis == 0:
         print dis
         _root.after_idle(do_idle)
#        _root.after(100, do_idle)

def do_connector(dis, fd):
    global _root
    print "python do_connector; dis=", dis, "fd=", fd
    if not _root:
        _root = Tkinter._default_root
        print "set _root", _root
    if fd >= 0:
        if dis == 0:
            _tkinter.createfilehandler(fd, _tkinter.READABLE, do_pending)
            print "file handle created"
        else:
            _tkinter.deletefilehandler(fd)
            print "file handle deleted"

def do_keyhandler(msg):
    print "python do_keyhandler"
    print msg
    # should evaluate msg as python command here

""" 
Initialization for Tkinter cooperating with Gist
"""

#  .. Make very sure gist has not stolen PyOS_InputHook
#     -- probably possible for this to be insufficient

gist.pyg_unhook()

#  .. can set this explicitly if Tkinter._default_root is not set
#     e.g.- if running IDLE (or anything else that uses PyShell):
#   import tkgist
#   tkgist._root = PyShell.root
#   from gist import *

print "_root, _refresh", _root, _refresh
gist.pyg_register(do_connector, do_keyhandler);



import Numeric
import cPickle

def objsave(file, allglobals, *args):
    """Pickle the part of a dictionary containing the argument list
    into file string.

    Syntax:  objsave(file, globals(), obj1, obj2, ... )
    """
    fid = open(file,'w')
    savedict = {}
    for key in allglobals.keys():
        inarglist = 0
        for obj in args:
            if allglobals[key] is obj:
                inarglist = 1
                break
        if inarglist:
            savedict[key] = obj
    cPickle.dump(savedict,fid,1)
    fid.close()
        
def objload(file, allglobals):
    """Load a previously pickled dictionary and insert into given dictionary.

    Syntax:  objload(file, globals())
    """
    fid = open(file,'r')
    savedict = cPickle.load(fid)
    allglobals.update(savedict)
    fid.close()



""" Test functions for basic module

"""

import unittest

import sys
from scipy_test.testing import *
set_package_path()
import scipy_base;reload(scipy_base)
from scipy_base import *
del sys.path[0]

##################################################

val = limits.double_resolution


def get_mat(n):
    data = arange(n)
    data = add.outer(data,data)
    return data

class test_eye(unittest.TestCase):
    def check_basic(self):
        assert_equal(eye(4),array([[1,0,0,0],
                                   [0,1,0,0],
                                   [0,0,1,0],
                                   [0,0,0,1]]))
        assert_equal(eye(4,typecode='f'),array([[1,0,0,0],
                                                [0,1,0,0],
                                                [0,0,1,0],
                                                [0,0,0,1]],'f'))
    def check_diag(self):
        assert_equal(eye(4,k=1),array([[0,1,0,0],
                                       [0,0,1,0],
                                       [0,0,0,1],
                                       [0,0,0,0]]))
        assert_equal(eye(4,k=-1),array([[0,0,0,0],
                                        [1,0,0,0],
                                        [0,1,0,0],
                                        [0,0,1,0]]))
    def check_2d(self):
        assert_equal(eye(4,3),array([[1,0,0],
                                     [0,1,0],
                                     [0,0,1],
                                     [0,0,0]]))
        assert_equal(eye(3,4),array([[1,0,0,0],
                                     [0,1,0,0],
                                     [0,0,1,0]]))        
    def check_diag2d(self):
        assert_equal(eye(3,4,k=2),array([[0,0,1,0],
                                         [0,0,0,1],
                                         [0,0,0,0]]))
        assert_equal(eye(4,3,k=-2),array([[0,0,0],
                                          [0,0,0],
                                          [1,0,0],
                                          [0,1,0]]))

class test_diag(unittest.TestCase):
    def check_vector(self):
        vals = (100*arange(5)).astype('l')
        b = zeros((5,5))
        for k in range(5):
            b[k,k] = vals[k]
        assert_equal(diag(vals),b)
        b = zeros((7,7))
        c = b.copy()
        for k in range(5):
            b[k,k+2] = vals[k]
            c[k+2,k] = vals[k]
        assert_equal(diag(vals,k=2), b)
        assert_equal(diag(vals,k=-2), c)

    def check_matrix(self):
        vals = (100*get_mat(5)+1).astype('l')
        b = zeros((5,))
        for k in range(5):
            b[k] = vals[k,k]
        assert_equal(diag(vals),b)
        b = b*0
        for k in range(3):
            b[k] = vals[k,k+2]
        assert_equal(diag(vals,2),b[:3])
        for k in range(3):
            b[k] = vals[k+2,k]
        assert_equal(diag(vals,-2),b[:3])

class test_fliplr(unittest.TestCase):
    def check_basic(self):
        self.failUnlessRaises(ValueError, fliplr, ones(4))        
        self.failUnlessRaises(ValueError, fliplr, ones((4,3,2)))
        a = get_mat(4)
        b = a[:,::-1]
        assert_equal(fliplr(a),b)
        a = [[0,1,2],
             [3,4,5]]
        b = [[2,1,0],
             [5,4,3]]
        assert_equal(fliplr(a),b)

class test_flipud(unittest.TestCase):
    def check_basic(self):
        self.failUnlessRaises(ValueError, flipud, ones(4))
        self.failUnlessRaises(ValueError, flipud, ones((4,3,2)))
        a = get_mat(4)
        b = a[::-1,:]
        assert_equal(flipud(a),b)
        a = [[0,1,2],
             [3,4,5]]
        b = [[3,4,5],
             [0,1,2]]
        assert_equal(flipud(a),b)

class test_rot90(unittest.TestCase):
    def check_basic(self):
        self.failUnlessRaises(ValueError, rot90, ones(4))
        self.failUnlessRaises(ValueError, rot90, ones((4,3,2)))

        a = [[0,1,2],
             [3,4,5]]
        b1 = [[2,5],
              [1,4],
              [0,3]]
        b2 = [[5,4,3],
              [2,1,0]]
        b3 = [[3,0],
              [4,1],
              [5,2]]
        b4 = [[0,1,2],
              [3,4,5]]

        for k in range(-3,13,4):
            assert_equal(rot90(a,k=k),b1)
        for k in range(-2,13,4):
            assert_equal(rot90(a,k=k),b2)
        for k in range(-1,13,4):
            assert_equal(rot90(a,k=k),b3)
        for k in range(0,13,4):
            assert_equal(rot90(a,k=k),b4)

if __name__ == "__main__":
    ScipyTest('scipy_base.matrix_base').run()



#!/usr/bin/env python
"""
Installing scipy:
  python setup.py install
  python setup.py fc_config --fcompiler=Gnu install

Building extensions in-place:
  python setup.py build_src build_ext --inplace

Creating scipy distribution:
  python setup.py sdist -f            #-> Scipy_complete
  python setup.py sdist_packagers -f  #-> Scipy and Scipy_core
"""

import os
import sys
from glob import glob

# Use 'setup.py sdist_packagers -f' to create separate tar-balls
# for Scipy and Scipy_core.
if 'sdist_packagers' in sys.argv:
    sys.argv[sys.argv.index('sdist_packagers')] = 'sdist'
    command_sdist = 1
elif 'bdist_rpm' in sys.argv:
    command_sdist = 1
else:
    command_sdist = 0

# Note that when running bdist_rpm, scipy_core directory does
# not exist. So, scipy_distutils must be installed before
# running bdist_rpm.
sys.path.insert(0,'scipy_core')
try:
    import scipy_distutils
    # Declare all scipy_distutils related imports here.
    from scipy_distutils.misc_util import default_config_dict
    from scipy_distutils.misc_util import get_path, merge_config_dicts
    from scipy_distutils.misc_util import get_subpackages, generate_config_py
    from scipy_distutils.core import setup, Extension
finally:
    del sys.path[0]

#-------------------------------

def setup_package(ignore_packages=[]):

    if command_sdist and os.path.isdir('scipy_core'):
        # Applying the same commands to scipy_core.
        # Results can be found in scipy_core directory.
        c = '%s %s %s' % (sys.executable,
                          os.path.abspath(os.path.join('scipy_core','setup.py')),
                          ' '.join(sys.argv[1:]))
        print c
        s = os.system(c)
        assert not s,'failed on scipy_core'

    old_path = os.getcwd()
    local_path = os.path.dirname(os.path.abspath(sys.argv[0]))

    os.chdir(local_path)
    sys.path.insert(0,os.path.join(local_path,'Lib'))
    # setup files of subpackages require scipy_core:
    sys.path.insert(0,os.path.join(local_path,'scipy_core'))
    try:
        
        #from scipy_version import scipy_version
        scipy_version = '0.3.2'

        packages_path = ['Lib']
        if command_sdist:
            name = 'SciPy'
        else:
            name = 'SciPy_complete'
            packages_path.append('scipy_core')
        config_list = [{'name': name,
                        'packages':['scipy','scipy.tests'],
                        'package_dir':
                        {'scipy':'Lib',
                         'scipy.tests':os.path.join('Lib','tests')}}]

        for d in packages_path:
            config_list += get_subpackages(os.path.join(local_path,d),
                                           parent = 'scipy',
                                           parent_path=local_path,
                                           ignore_packages = ignore_packages)
        config_dict = merge_config_dicts(config_list)

        ext = Extension(name='scipy.config',
                        sources=[generate_config_py])
        config_dict['ext_modules'].append(ext)

        print 'SciPy Version %s' % scipy_version
        setup( version = scipy_version,
               maintainer = "SciPy Developers",
               maintainer_email = "scipy-dev@scipy.org",
               description = "Scientific Algorithms Library for Python",
               license = "SciPy License (BSD Style)",
               url = "http://www.scipy.org",
               **config_dict
              )

    finally:
        del sys.path[0]
        del sys.path[0]
        os.chdir(old_path)

if __name__ == "__main__":
    ignore_packages = [
        # list of package names that will not be built.
        ]
    setup_package(ignore_packages)



import types
import  time
from random import *
SymbolError = 'Symbol Error'
DepthError = 'Depth Error'


class language:
	max_depth = 20
	dont_cross = ['ST']  # dont perform crossovers at the start symbol node
	def __init__(self,lang):
		self.lang = lang
		self.dsc = 0
	def generate(self):	
		new = self._gen()
		new.generate_symbol_table()
		return new
	def _gen(self, cur_sym = 'ST', active_node = None, depth = 0):
		#short circuit really large trees
		if(active_node and depth > self.max_depth):
			self.dsc = self.dsc + 1
			active_node.root().delete_circulars()
			raise DepthError
		depth = depth + 1	
		#reset derivation symbol
		if(str(cur_sym) == 'ST' or self.derive_type == ''):
			if(type(cur_sym) == types.StringType):
				self.derive_type = cur_sym
			else:	self.derive_type = cur_sym.node_type
		new_active_node = active_node
		if type(cur_sym) == types.StringType:
			if self.lang.has_key(cur_sym):
				rule = choice(self.lang[cur_sym])
				for sym in rule:
					new_active_node = self._gen(sym,new_active_node,depth)
			else:
				active_node.root().delete_circulars()
				raise SymbolError, "non-terminal symbol not found:%s" % cur_sym
	
		else:
			parent = new_active_node
			new_active_node = cur_sym.create(parent)
			new_active_node.derive_type = self.derive_type
			self.derive_type = ''
			if parent:
				parent.add_child(new_active_node)
			while new_active_node.filled() and new_active_node.get_parent():	
				new_active_node =new_active_node.get_parent()
		return new_active_node

		



"""
check_var_in -- tests whether a variable is passed in correctly
                and also if the passed in variable can be reassigned
check_var_local -- tests wheter a variable is passed in , modified,
                   and returned correctly in the local_dict dictionary
                   argument
check_return -- test whether a variable is passed in, modified, and
                then returned as a function return value correctly
"""
import unittest

from scipy_test.testing import *
set_package_path()
from weave import ext_tools, wx_spec
restore_path()

import wxPython
import wxPython.wx

class test_wx_converter(unittest.TestCase):    
    def check_type_match_string(self,level=5):
        s = wx_spec.wx_converter()
        assert(not s.type_match('string') )
    def check_type_match_int(self,level=5):
        s = wx_spec.wx_converter()        
        assert(not s.type_match(5))
    def check_type_match_float(self,level=5):
        s = wx_spec.wx_converter()        
        assert(not s.type_match(5.))
    def check_type_match_complex(self,level=5):
        s = wx_spec.wx_converter()        
        assert(not s.type_match(5.+1j))
    def check_type_match_complex(self,level=5):
        s = wx_spec.wx_converter()        
        assert(not s.type_match(5.+1j))
    def check_type_match_wxframe(self,level=5):
        s = wx_spec.wx_converter()
        f=wxPython.wx.wxFrame(wxPython.wx.NULL,-1,'bob')        
        assert(s.type_match(f))
        
    def check_var_in(self,level=5):
        mod = ext_tools.ext_module('wx_var_in',compiler='msvc')
        a = wxPython.wx.wxFrame(wxPython.wx.NULL,-1,'bob')        
        code = """
               a->SetTitle(wxString("jim"));
               """
        test = ext_tools.ext_function('test',code,['a'],locals(),globals())
        mod.add_function(test)
        mod.compile()
        import wx_var_in
        b=a
        wx_var_in.test(b)
        assert(b.GetTitle() == "jim")
        try:
            b = 1.
            wx_var_in.test(b)
        except TypeError:
            pass
        try:
            b = 1
            wx_var_in.test(b)
        except TypeError:
            pass
            
    def no_check_var_local(self,level=5):
        mod = ext_tools.ext_module('wx_var_local')
        a = 'string'
        var_specs = ext_tools.assign_variable_types(['a'],locals())
        code = 'a=Py::String("hello");'
        test = ext_tools.ext_function('test',var_specs,code)
        mod.add_function(test)
        mod.compile()
        import wx_var_local
        b='bub'
        q={}
        wx_var_local.test(b,q)
        assert(q['a'] == 'hello')
    def no_check_return(self,level=5):
        mod = ext_tools.ext_module('wx_return')
        a = 'string'
        var_specs = ext_tools.assign_variable_types(['a'],locals())
        code = """
               a= Py::wx("hello");
               return_val = Py::new_reference_to(a);
               """
        test = ext_tools.ext_function('test',var_specs,code)
        mod.add_function(test)
        mod.compile()
        import wx_return
        b='bub'
        c = wx_return.test(b)
        assert( c == 'hello')

if __name__ == "__main__":
    ScipyTest('weave.wx_spec').run()



"""
Discrete Fourier Transforms - basic.py 
"""
# Created by Pearu Peterson, August,September 2002

__all__ = ['fft','ifft','fftn','ifftn','rfft','irfft',
           'fft2','ifft2']

from scipy_base import asarray, zeros, swapaxes
import _fftpack as fftpack

import atexit
atexit.register(fftpack.destroy_zfft_cache)
atexit.register(fftpack.destroy_zfftnd_cache)
atexit.register(fftpack.destroy_drfft_cache)
del atexit

def _fix_shape(x, n, axis):
    """ Internal auxiliary function for _raw_fft, _raw_fftnd."""
    s = list(x.shape)
    if s[axis] > n:
        index = [slice(None)]*len(s)
        index[axis] = slice(0,n)
        x = x[index]
    else:
        index = [slice(None)]*len(s)
        index[axis] = slice(0,s[axis])
        s[axis] = n
        z = zeros(s,x.typecode())
        z[index] = x
        x = z
    return x


def _raw_fft(x, n, axis, direction, overwrite_x, work_function):
    """ Internal auxiliary function for fft, ifft, rfft, irfft."""
    if n is None:
        n = x.shape[axis]
    elif n != x.shape[axis]:
        x = _fix_shape(x,n,axis)
        overwrite_x = 1
    if axis == -1 or axis == len(x.shape)-1:
        r = work_function(x,n,direction,overwrite_x=overwrite_x)
    else:
        x = swapaxes(x, axis, -1)
        r = work_function(x,n,direction,overwrite_x=overwrite_x)
        r = swapaxes(r, axis, -1)
    return r


def fft(x, n=None, axis=-1, overwrite_x=0):
    """ fft(x, n=None, axis=-1, overwrite_x=0) -> y

    Return discrete Fourier transform of arbitrary type sequence x.

    The returned complex array contains
      [y(0),y(1),..,y(n/2-1),y(-n/2),...,y(-1)]        if n is even
      [y(0),y(1),..,y((n-1)/2),y(-(n-1)/2),...,y(-1)]  if n is odd
    where
      y(j) = sum[k=0..n-1] x[k] * exp(-sqrt(-1)*j*k* 2*pi/n)
      j = 0..n-1
    Note that y(-j) = y(n-j).

    Optional input:
      n
        Defines the length of the Fourier transform. If n is not
        specified then n=x.shape[axis] is set. If n<x.shape[axis],
        x is truncated. If n>x.shape[axis], x is zero-padded.
      axis
        The trasnform is applied along the given axis of the input
        array (or the newly constructed array if n argument was used).
      overwrite_x
        If set to true, the contents of x can be destroyed.

    Notes:
      y == fft(ifft(y)) within numerical accuracy.
    """
    tmp = asarray(x)
    t = tmp.typecode()
    if t=='D':
        overwrite_x = overwrite_x or (tmp is not x and not \
                                      hasattr(x,'__array__'))
        work_function = fftpack.zfft
    elif t=='F':
        raise NotImplementedError
    else:
        overwrite_x = 1
        work_function = fftpack.zrfft

    #return _raw_fft(tmp,n,axis,1,overwrite_x,work_function)
    if n is None:
        n = tmp.shape[axis]
    elif n != tmp.shape[axis]:
        tmp = _fix_shape(tmp,n,axis)
        overwrite_x = 1

    if axis == -1 or axis == len(tmp.shape) - 1:
        return work_function(tmp,n,1,0,overwrite_x)

    tmp = swapaxes(tmp, axis, -1)
    tmp = work_function(tmp,n,1,0,overwrite_x)
    return swapaxes(tmp, axis, -1)

def ifft(x, n=None, axis=-1, overwrite_x=0):
    """ ifft(x, n=None, axis=-1, overwrite_x=0) -> y

    Return inverse discrete Fourier transform of arbitrary type
    sequence x.

    The returned complex array contains
      [y(0),y(1),...,y(n-1)]
    where
      y(j) = 1/n sum[k=0..n-1] x[k] * exp(sqrt(-1)*j*k* 2*pi/n)

    Optional input: see fft.__doc__
    """
    tmp = asarray(x)
    t = tmp.typecode()
    if t=='D':
        overwrite_x = overwrite_x or (tmp is not x and not \
                                      hasattr(x,'__array__'))
        work_function = fftpack.zfft
    elif t=='F':
        raise NotImplementedError
    else:
        overwrite_x = 1
        work_function = fftpack.zrfft

    #return _raw_fft(tmp,n,axis,-1,overwrite_x,work_function)
    if n is None:
        n = tmp.shape[axis]
    elif n != tmp.shape[axis]:
        tmp = _fix_shape(tmp,n,axis)
        overwrite_x = 1

    if axis == -1 or axis == len(tmp.shape) - 1:
        return work_function(tmp,n,-1,1,overwrite_x)

    tmp = swapaxes(tmp, axis, -1)
    tmp = work_function(tmp,n,-1,1,overwrite_x)
    return swapaxes(tmp, axis, -1)


def rfft(x, n=None, axis=-1, overwrite_x=0):
    """ rfft(x, n=None, axis=-1, overwrite_x=0) -> y

    Return discrete Fourier transform of real sequence x.

    The returned real arrays contains
      [y(0),Re(y(1)),Im(y(1)),...,Re(y(n/2))]              if n is even
      [y(0),Re(y(1)),Im(y(1)),...,Re(y(n/2)),Im(y(n/2))]   if n is odd
    where
      y(j) = sum[k=0..n-1] x[k] * exp(-sqrt(-1)*j*k* 2*pi/n)
      j = 0..n-1
    Note that y(-j) = y(n-j).

    Optional input:
      n
        Defines the length of the Fourier transform. If n is not
        specified then n=x.shape[axis] is set. If n<x.shape[axis],
        x is truncated. If n>x.shape[axis], x is zero-padded.
      axis
        The transform is applied along the given axis of the input
        array (or the newly constructed array if n argument was used).
      overwrite_x
        If set to true, the contents of x can be destroyed.

    Notes:
      y == rfft(irfft(y)) within numerical accuracy.
    """
    tmp = asarray(x)
    t = tmp.typecode()
    if t in 'DF':
        raise TypeError,"1st argument must be real sequence"
    work_function = fftpack.drfft
    return _raw_fft(tmp,n,axis,1,overwrite_x,work_function)


def irfft(x, n=None, axis=-1, overwrite_x=0):
    """ irfft(x, n=None, axis=-1, overwrite_x=0) -> y

    Return inverse discrete Fourier transform of real sequence x.
    The contents of x is interpreted as the output of rfft(..)
    function.

    The returned real array contains
      [y(0),y(1),...,y(n-1)]
    where for n is even
      y(j) = 1/n (sum[k=1..n/2-1] (x[2*k-1]+sqrt(-1)*x[2*k])
                                   * exp(sqrt(-1)*j*k* 2*pi/n)
                  + c.c. + x[0] + (-1)**(j) x[n-1])
    and for n is odd
      y(j) = 1/n (sum[k=1..(n-1)/2] (x[2*k-1]+sqrt(-1)*x[2*k])
                                   * exp(sqrt(-1)*j*k* 2*pi/n)
                  + c.c. + x[0])
    c.c. denotes complex conjugate of preceeding expression.

    Optional input: see rfft.__doc__
    """
    tmp = asarray(x)
    t = tmp.typecode()
    if t in 'DF':
        raise TypeError,"1st argument must be real sequence"
    work_function = fftpack.drfft
    return _raw_fft(tmp,n,axis,-1,overwrite_x,work_function)


def _raw_fftnd(x, s, axes, direction, overwrite_x, work_function):
    """ Internal auxiliary function for fftnd, ifftnd."""
    if s is None:
        s = x.shape
    s = tuple(s)
    if s!=x.shape:
        assert len(s)<=len(x.shape)
        for i in range(-len(s),0):
            if x.shape[i]!=s[i]:
                x = _fix_shape(x,s[i],i)
    if axes is None:
        return work_function(x,s,direction,overwrite_x=overwrite_x)

    #XXX: should we allow/check for repeated indices in axes?
    # If allowed then using it has an effect of reducing the shape
    # implicitly. 
    s = [x.shape[i] for i in axes]
    s = [1]*(len(x.shape)-len(s)) + s
    swaps = []
    state = range(-len(s),0)
    for i in range(-1,-len(axes)-1,-1):
        j = state[axes[i]]
        if i==j: continue
        state[i],state[j]=state[j],state[i]
        swaps.append((j,i))
        x = swapaxes(x, j,i)
    r = work_function(x,s,direction,overwrite_x=overwrite_x)
    swaps.reverse()
    for i,j in swaps:
        r = swapaxes(r,i,j)
    return r


def fftn(x, shape=None, axes=None, overwrite_x=0):
    """ fftn(x, shape=None, axes=None, overwrite_x=0) -> y
    
    Return multi-dimensional discrete Fourier transform of arbitrary
    type sequence x.

    The returned array contains

      y[j_1,..,j_d] = sum[k_1=0..n_1-1, ..., k_d=0..n_d-1]
         x[k_1,..,k_d] * prod[i=1..d] exp(-sqrt(-1)*2*pi/n_i * j_i * k_i)

    where d = len(x.shape) and n = x.shape.
    Note that y[..., -j_i, ...] = y[..., n_i-j_i, ...].

    Optional input:
      shape
        Defines the shape of the Fourier transform. If shape is not
        specified then shape=take(x.shape,axes).
        If shape[i]>x.shape[i] then the i-th dimension is padded with
        zeros. If shape[i]<x.shape[i], then the i-th dimension is
        truncated to desired length shape[i].
      axes
        The transform is applied along the given axes of the input
        array (or the newly constructed array if shape argument was
        used).
      overwrite_x
        If set to true, the contents of x can be destroyed.

    Notes:
      y == fftn(ifftn(y)) within numerical accuracy.
    """
    tmp = asarray(x)
    t = tmp.typecode()
    if t=='D':
        overwrite_x = overwrite_x or (tmp is not x and not \
                                      hasattr(x,'__array__'))
        work_function = fftpack.zfftnd
    elif t=='F':
        raise NotImplementedError
    else:
        overwrite_x = 1
        work_function = fftpack.zfftnd
    return _raw_fftnd(tmp,shape,axes,1,overwrite_x,work_function)


def ifftn(x, shape=None, axes=None, overwrite_x=0):
    """ ifftn(x, s=None, axes=None, overwrite_x=0) -> y
    
    Return inverse multi-dimensional discrete Fourier transform of
    arbitrary type sequence x.

    The returned array contains

      y[j_1,..,j_d] = 1/p * sum[k_1=0..n_1-1, ..., k_d=0..n_d-1]
         x[k_1,..,k_d] * prod[i=1..d] exp(sqrt(-1)*2*pi/n_i * j_i * k_i)

    where d = len(x.shape), n = x.shape, and p = prod[i=1..d] n_i.

    Optional input: see fftn.__doc__
    """
    tmp = asarray(x)
    t = tmp.typecode()
    if t=='D':
        overwrite_x = overwrite_x or (tmp is not x and not \
                                      hasattr(x,'__array__'))
        work_function = fftpack.zfftnd
    elif t=='F':
        raise NotImplementedError
    else:
        overwrite_x = 1
        work_function = fftpack.zfftnd
    return _raw_fftnd(tmp,shape,axes,-1,overwrite_x,work_function)


def fft2(x, shape=None, axes=(-2,-1), overwrite_x=0):
    """ fft2(x, shape=None, axes=(-2,-1), overwrite_x=0) -> y
    
    Return two-dimensional discrete Fourier transform of
    arbitrary type sequence x.

    See fftn.__doc__ for more information.
    """
    return fftn(x,shape,axes,overwrite_x)


def ifft2(x, shape=None, axes=(-2,-1), overwrite_x=0):
    """ ifft2(x, shape=None, axes=(-2,-1), overwrite_x=0) -> y
    
    Return inverse two-dimensional discrete Fourier transform of
    arbitrary type sequence x.

    See ifftn.__doc__ for more information.
    """
    return ifftn(x,shape,axes,overwrite_x)



"""
Genes are the most basic building block in this genetic algorithm library.
A gene represents a particular trait of an individual solution.  Mutliple genes are
combined together to form a genome.  The entire genome represents a solution
to the problem being solved.

This module contains the base gene class from which all genes are derived.
It also has two gene classes, list_gene and float_gene, that are commonly used.
If you need to create your own gene class, derive it from the class gene.

history:
12/31/98 documentation added ej
"""

from ga_util import *
import scipy.stats as rv
from random import random
import copy

class gene:
    """
    Genes are the most basic building block in this genetic algorithm library.
    A gene represents a particular trait of an individual solution.  The gene class
    contains the current value of a gene object.  It also knows all the possible
    values that the gene can legally have (the allele set).  The gene itself does not 
    know how to initialize itself to a value or mutate to another value.  Instead, it is 
    closely coupled with initializer and mutator classes that perform these duties.
    By changing the initialization and mutation classes, the behavior of a gene can be 
    altered without actually having to create a new gene class.
    
    This class is never actually instantiated.  Instead it is a base class from which 
    specific gene classes are derived.  The duties of the derived class are 
    pretty limited because this class defines almost all the necessary methods.  The 
    derived class must define the allele set for the gene.  It usually will specify 
    an __init__() method, and possibly some other methods specific to the derived
    gene.  It is also necessary to specifiy the initializer and mutator classes 
    to operate on the new gene class
    
    There are several attributes for the class gene:
    
    mutation_rate -- Every gene has a possibility of mutating each generation.
                     This is a value between 0 and 1 that specifies, in percent, 
                     how often the gene mutates.  mutation_rate can either be
                     set for the entire gene class or on a gene by gene basis.
                     Use the set_mutation() function to change the rate
    mr_bounds -- Sometimes it is useful for the mutation rate of the gene to
                 adapt over the course of evolution.  This 2 element tuple specifies
                 the upper and lower bounds that the mutation rate can have.
                 note:  I haven't really messed with this much and think there
                        is probably a better approach
    mutator -- a mutator object (instantiation of a mutator class) that is used 
               to mutate the gene
    initializer -- an intializer object used to initialize the gene
    
    Take a look at float_gene and list_gene to see examples of derived gene classes.
    """
    mr_bounds = (0,.1)
    mutation_rate = .03
    mutator = None
    initializer = None
    is_gene = 1
    def clone(self): 
        """Makes a shallow copy of the object.  override if you need more specialized behavior
        """
        return shallow_clone(self)
    def replicate(self,cnt): 
        """Returns a list with cnt copies of this object in it
        """
        return map(lambda x: x.clone(),[self]*cnt)
    def initialize(self):
        """Calls the initializer objects evaluate() function to initialize the gene 
        """
        self._value = self.initializer.evaluate(self)
        return self.value()
    def set_mutation(self,mrate):
        """
        Set the mutation rate of the gene.
        
            Arguments:

              mrate -- can be one of the following:
              
                * a number between 0 and 1 - sets the mutation rate of the gene to a specific value.
                * "gene" - use the mutation rate set in the class definition for this gene.
                * "adapt" - the mutation rate for the gene is chosen randomly from the range mr_bounds
        """
        if(mrate=='gene'): 
            try: del self.mutation_rate #remove local mrates and use gene classes mrate
            except AttributeError: pass
        elif(mrate=='adapt'): 
            self.mutation_rate = rv.uniform(self.mr_bounds[0],self.mr_bounds[1])[0]
        else: 
            self.__class__.mutation_rate = mrate

    def mutate(self):
        """
            Returns 1 if gene mutated, 0 otherwise.
                
            Calls the **mutator.evaluate()** function to mutate the gene 
            mutation_rate of the time. Otherwise, it does nothing.      
        """ 
        #inlined 'flip_coin' for speed
        if random() < self.mutation_rate: 
            self._value = self.mutator.evaluate(self)
            return 1
        return 0
    def value(self):
        """Return the current value of the gene. """ 
        try: 
            return self._value
        except AttributeError: 
            raise GAError, 'gene not initialized'
    def set_value(self,x):
        """ Set the value of a gene. NO CHECKING!!!
            Don't assign an incompatible value.
        """ 
        self._value = x
    def __repr__(self): 
        try: return `self.value()`
        except GAError: return 'gene not initialized'
    def __add__(self, other):
        try: return self.value() + other.value()
        except AttributeError: return self.value() + other
    __radd__ = __add__
    def __mul__(self, other):
        try: return self.value() * other.value()
        except AttributeError: return self.value() * other
    __rmul__ = __mul__
    def __sub__(self, other):
        try: return self.value() - other.value()
        except AttributeError: return self.value() - other
    def __rsub__(self, other):
        try: return other.value() - self.value()
        except AttributeError: return other - self.value()
    def __div__(self, other):
        try: return self.value() / other.value()
        except: return self.value() / other
    def __rdiv__(self, other):
        try: return other.value() / self.value()
        except AttributeError: return other / self.value()
    def __float__(self): return float(self.value())
    def __complex__(self): return float(self.value())   
    def __neg__(self): return -self.value()
    def __cmp__(self, other):
        try: 
            if self.__class__ == other.__class__ and self.__dict__ == other.__dict__: return 0
        except AttributeError: pass
        v1 = self.value()
        try: v2 = other.value()
        except AttributeError: v2 = other
        return cmp(v1,v2)
        
class list_gene_uniform_mutator:
    """ 
    This class randomly chooses a new gene value from the allele set
    in a list_gene.  It is also useful as an initializer for list_gene.
    """
    def evaluate(self,gene): 
        """ return a randomly chosen value from the genes allele set """
        return rv.choice(gene.allele_set)
class list_gene_gaussian_mutator:
    """ 
    This class chooses a new gene value from the allele set
    in a list_gene.  The new value is chosen from a gaussian 
    distributed distance away from the current values location in the 
    allele set list.  The mutated value is never equal to the current
    gene value.  The dev_width is the standard deviation of the gaussian
    distribution as a percentage of the length of the list.
        
    As an example, suppose a list_gene has the allele_set [0,1,2,3,4,5,6,7,8,9].
    There are 10 entries in this list.  If the dev_width is .1 (the default), 
    then there is a 65% chance the new value will either be 1 position away from
    the current value.  If the current value is 4, then the new value will be
    3 or 5 66% of the time, 2 or 6 29% of the time, and so on based on a gaussian
    distribution.
        
    If the newly chosen index falls outside of the range of the list (for example 
    -1), then a new value is chosen until the value falls inside the lists range.
    The index is NOT truncated to the bottom or top index in the range.    
    """
    def __init__(self,dev_width = .1):
        """Arguments:
            dev_width -- a value between 0 and 1 that specifies the standard
            deviation as a percentage of the length of the list_gene's 
            allele set.
        """
        self.dev_width = dev_width
    def evaluate(self,gene):
        """ return a new value from the genes allele set """
        size = len(gene.allele_set)
        if size == 1: return gene.allele_set[0]
        w = self.dev_width * size
        old = gene.index()
        new = -1; f = -1
        while not (0 <= new < size):
            f = rv.norm(old,w)[0] 
            new = round(f)
            if(old == new and f > new): new = new + 1
            if(old == new and f < new): new = new - 1
        return gene.allele_set[int(new)] 
class list_gene_walk_mutator:
    """ 
      This class chooses a new gene value from the allele set
      in a list_gene.  The newly chosen value is +/-1 element
    in the allele_set from the current gene value. 
    This is like a random walk across the allele_set
    """
    def evaluate(self,gene):
        old = gene.index()
        move = rv.choice((-1,1))
        return gene.allele_set[old + move]
    
class list_gene(gene):
    """
    The value of a list_gene is chosen from a list of
    possible values - the allele_set.
    For example, the gene could be used to represent a
    mathematical oeprator.  Here the allele_set might be
    ['+','-','*','/'].  The list could just as easily be
    a list of numbers (ie. standard capacitor values),
    strings, or anything else.
    
    The default mutator is a gaussian mutator and the 
    default initializer randomly chooses a value from the
    allele_set.
    """
    gaussian_mutator = list_gene_gaussian_mutator
    uniform_mutator = list_gene_uniform_mutator
    walk_mutator = list_gene_walk_mutator
    mutator = gaussian_mutator()
    initializer = uniform_mutator()
    def __init__(self, allele_set): self.allele_set = allele_set
    def index(self,*val):
        """set or retreive a specific value from the allele_set"""
        if len(val): self._value = self.allele_set[val[0]]
        return self.allele_set.index(self.value())  

class list2_gene(list_gene):
    """
    this is something like we'll do to add part variance to capacitor
    and resistor values during evaluation
    """
    func = nop
    def value(self): return func(self._value)
    def __repr__(self): return `self._value` #???

class float_gene_uniform_mutator:
    """ randomly choose a value within the float_gene's bounds"""
    def evaluate(self,gene):
        bounds=gene.bounds
        new =rv.uniform(bounds[0], bounds[1]-bounds[0] ).rvs()[0]
        return new

class float_gene_gaussian_mutator:
    """ 
    chooses a new value for a float_gene with gaussian 
    shaped distribution around the current value.  
    
    dev_width -- a value between 0 and 1.  It is the standard
    deviation for the gaussian distribution as a percentage
    of the float_gene's range.  For example:  If the genes bounds
    are (0,10) and dev_width is .1, then the standard deviation
    is 1.
    """

    def __init__(self,dev_width = .1):
        self.dev_width = dev_width
    def evaluate(self,gene):
        dev = (gene.bounds[1]-gene.bounds[0]) * self.dev_width
        new = gene.bounds[1]
#       while not (gene.bounds[0] <= new < gene.bounds[1]):
#           new = rv.norm(gene.value(),dev)[0]
#       new = rv.norm(gene.value(),dev)[0]
        #get the _value explicitly so mutator will work for log_float also
        new = rv.norm(gene._value,dev).rvs()[0]
        if new > gene.bounds[1]: new = gene.bounds[1]
        if new < gene.bounds[0]: new = gene.bounds[0]
        return new

class float_gene(gene):
    """
    A float_gene is a gene that takes on a floating point value
    between some upper and lower bounds.
    
    The default mutator is a gaussian mutator and the 
    default initializer randomly chooses a value from within
    the upper and lower bounds.
    
    bounds -- A 2 element tuple that specifies the lower and upper
    bounds for the gene.
    """
    gaussian_mutator = float_gene_gaussian_mutator
    uniform_mutator = float_gene_uniform_mutator
    mutator = gaussian_mutator()
    initializer = uniform_mutator()
    def __init__(self,bounds):
        if len(bounds) !=2: raise GAError, 'float_gene: init expects a 2 element tuple of the fomr (min,max)'
        self.bounds = bounds
    def set_value(self,x):
        """ Set the value of a gene. 
            Convertthe value to a float first!
        """ 
        self._value = float(x)

from Numeric import *
from scipy_base.fastumath import *
class log_float_gene(float_gene):
    def __init__(self,bounds):
        if len(bounds) !=2: raise GAError, 'float_gene: init expects a 2 element tuple of the fomr (min,max)'
        self.bounds = log10(array(bounds))
    def value(self):
        """Return the current value of the gene. """ 
        try: return 10.**(self._value)
        except AttributeError: raise GAError, 'gene not initialized'
        
class frozen:
    """frozen is a gene that always maintains the same value.
    """
    def __init__(self,val): self._value = val
    def initialize(self): pass
    def set_mutation(self,mrate): pass
    def mutate(self): pass
    def value(self) : return self._value
    def clone(self): return shallow_clone(self)
    def __float__(self): return float(self._value)
    def __repr__(self): return `self._value`
    def __add__(self, other):
        try: return self._value + other.value()
        except AttributeError: return self._value + other
    __radd__ = __add__
    def __mul__(self, other):
        try: return self._value * other.value()
        except AttributeError: return self._value * other
    __rmul__ = __mul__
    def __sub__(self, other):
        try: return self._value - other.value()
        except AttributeError: return self._value - other
    def __rsub__(self, other):
        try: return other.value() - self._value
        except AttributeError: return other - self._value
    def __div__(self, other):
        try: return self._value / other.value()
        except: return self._value / other
    def __rdiv__(self, other):
        try: return other.value() / self._value
        except AttributeError: return other / self._value
    def __float__(self): return float(self._value)
    def __neg__(self): return -self._value
    def __cmp__(self, other):
        try: 
            if self.__class__ == other.__class__ and self.__dict__ == other.__dict__: return 0
        except AttributeError: pass
        v1 = self.value()
        try: v2 = other.value()
        except AttributeError: v2 = other
        return cmp(v1,v2)

# not sure why this has to be fully qualified, but things are failing otherwise.
# import tree       
from scipy.ga.tree import tree_node
class tree_gene(tree_node):
    mr_bounds = (0,.1)
    mutation_rate = .03
    model_properties = {}
    def __init__(self,child_count,node_type='',derive_type='', parent=None):
        tree_node.__init__(self,child_count,node_type,derive_type, parent)
    def initialize(self,propagate = 1):
        if propagate:
            for child in self._children: child.initialize()         
    def defaultize(self):
        for child in self._children: child.defaultize()     
    def set_mutation(self,mrate):
        if(mrate=='gene'): 
            try: del self.mutation_rate #remove local mrates and use gene classes mrate
            except AttributeError: pass
        elif(mrate=='adapt'): 
            self.mutation_rate = rv.uniform(self.mr_bounds[0],self.mr_bounds[1])[0]
        else: 
            self.__class__.mutation_rate = mrate
        for child in self._children: child.set_mutation(mrate)
                
    def mutate(self,propagate = 1):
        mutated = 0
        #if flip_coin(self.mutation_rate): pass # handle tree mutation
        if propagate:
            for child in self._children: 
                #careful with short circuit "or"
                mutated = child.mutate() or mutated
        return mutated

    def value(self):
        """Return the current value of the gene. """ 
        try: return self._value
        except AttributeError: raise GAError, 'gene not initialized'
    def set_value(self,x):
        """ Set the value of a gene. NO CHECKING!!!
            Don't assign an incompatible value.
        """ 
        self._value = x
